@@ -1,7 +1,7 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_whatever.txt
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+ License: See LICENSE_unlicense.txt
 ***************************************************************************/
 
 #include <iostream>
@@ -14,7 +14,7 @@
 
 int main(int argc, char** argv)
 {
-	Glib::thread_init();
+// 	Glib::thread_init();
 	Gtk::Main m(argc, argv);
 
 	// NOTE: Don't use long options (e.g. --info). Use short ones (e.g. -i),
