@@ -1,11 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
-#include <stdint.h>  // uint64_t
+#include <clocale>  // localeconv
 
+#include "hz/locale_tools.h"  // ScopedCLocale, locale_c_get().
+#include "hz/cstdint.h"  // uint64_t
 #include "hz/string_algo.h"  // string_*
 #include "hz/string_num.h"  // string_is_numeric, number_to_string
 #include "hz/format_unit.h"  // format_size
@@ -17,6 +19,7 @@
 #include "storage_property_colors.h"
 
 
+
 namespace {
 
 
@@ -147,23 +150,34 @@ bool SmartctlParser::parse_full(const std::string& full)
 
 	// version info
 
-	std::string version = parse_version(s);
-	if (version.empty()) {
+	std::string version, version_full;
+	if (!parse_version(s, version, version_full)) {
 		set_error_msg("Cannot extract smartctl version information.");
 		debug_out_warn("app", DBG_FUNC_MSG << "Cannot extract version information. Returning.\n");
 		return false;
 
 	} else {
-		StorageProperty p;
-		p.set_name("Smartctl version", "smartctl_version");
-		p.reported_value = version;
-		p.value_type = StorageProperty::value_type_string;
-		p.value_string = p.reported_value;
-		p.section = StorageProperty::section_info;  // add to info section
-		add_property(p);
+		{
+			StorageProperty p;
+			p.set_name("Smartctl version", "smartctl_version", "Smartctl Version");
+			p.reported_value = version;
+			p.value_type = StorageProperty::value_type_string;
+			p.value_string = p.reported_value;
+			p.section = StorageProperty::section_info;  // add to info section
+			add_property(p);
+		}
+		{
+			StorageProperty p;
+			p.set_name("Smartctl version", "smartctl_version_full", "Smartctl Version");
+			p.reported_value = version_full;
+			p.value_type = StorageProperty::value_type_string;
+			p.value_string = p.reported_value;
+			p.section = StorageProperty::section_info;  // add to info section
+			add_property(p);
+		}
 	}
 
-	if (!check_version(version)) {
+	if (!check_version(version, version_full)) {
 		set_error_msg("Incompatible smartctl version.");
 		debug_out_warn("app", DBG_FUNC_MSG << "Incompatible smartctl version. Returning.\n");
 		return false;
@@ -197,35 +211,38 @@ bool SmartctlParser::parse_full(const std::string& full)
 		section_start_pos = section_end_pos;
 	}
 
-	return status;
+	if (!status) {
+		set_error_msg("No ATA sections could be parsed.");
+		debug_out_warn("app", DBG_FUNC_MSG << "No ATA sections could be parsed. Returning.\n");
+		return false;
+	}
+
+	return true;
 }
 
 
 
 // Supply output of "smartctl --version" here.
-// returns empty string on failure. Non-unix newlines in s are ok.
-std::string SmartctlParser::parse_version(const std::string& s)
+// returns false on failure. Non-unix newlines in s are ok.
+bool SmartctlParser::parse_version(const std::string& s, std::string& version, std::string& version_full)
 {
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
 	// e.g. "smartctl version 5.37" or "smartctl 5.39"
-	pcrecpp::RE re("^smartctl (?:version )?([0-9][^ \\t\\n\\r]+)", re_options);
-	std::string version;
-
-	if (!re.PartialMatch(s, &version)) {
+	// "smartctl 5.39 2009-06-03 20:10" (cvs versions)
+	// "smartctl 5.39 2009-08-08 r2873" (svn versions)
+	if (!app_pcre_match("/^smartctl (?:version )?(([0-9][^ \\t\\n\\r]+)(?: [0-9 r:-]+)?)/mi", s, &version_full, &version)) {
 		debug_out_error("app", DBG_FUNC_MSG << "No smartctl version information found in supplied string.\n");
-		return std::string();
+		return false;
 	}
+	hz::string_trim(version_full);
 
-	return version;
+	return true;
 }
 
 
 
 
 // check that the version of smartctl output can be parsed with this parser.
-bool SmartctlParser::check_version(const std::string& version_str)
+bool SmartctlParser::check_version(const std::string& version_str, const std::string& version_full_str)
 {
 	// tested with 5.1-xx versions (1 - 18), and 5.[20 - 38].
 	// note: 5.1-11 (maybe others too) with scsi disk gives non-parsable output (why?).
@@ -249,10 +266,40 @@ bool SmartctlParser::check_version(const std::string& version_str)
 // Note: this property is present since 5.33.
 std::string SmartctlParser::parse_byte_size(const std::string& str, uint64_t& bytes, bool extended)
 {
-	// e.g. "500,107,862,016" bytes or "80'060'424'192 bytes" or "80 026 361 856 bytes"
-	// (locale-dependent?). added '.'-separated too, just in case.
-	std::string s = hz::string_replace_copy(hz::string_replace_copy(hz::string_trim_copy(str), ",", ""), "'", "");
-	s = hz::string_replace_copy(hz::string_replace_copy(hz::string_replace_copy(s, " bytes", ""), " ", ""), ".", "");
+	// E.g. "500,107,862,016" bytes or "80'060'424'192 bytes" or "80 026 361 856 bytes".
+	// French locale inserts 0xA0 as a separator (non-breaking space, _not_ a valid utf8 char).
+	// Added '.'-separated too, just in case.
+	// Smartctl uses system locale's thousands_sep explicitly.
+
+	// When launching smartctl, we use LANG=C for it, but it works only on POSIX.
+	// Also, loading smartctl output files from different locales doesn't really work.
+
+// 	debug_out_dump("app", "Size reported as: " << str << "\n");
+
+	std::vector<std::string> to_replace;
+	to_replace.push_back(" ");
+	to_replace.push_back("'");
+	to_replace.push_back(",");
+	to_replace.push_back(".");
+	to_replace.push_back(std::string(1, 0xa0));
+
+#ifdef _WIN32
+	// if current locale is C, then probably we didn't change it at application
+	// startup, so set it now (temporarily). Otherwise, just use the current locale's
+	// thousands separator.
+	{
+		std::string old_locale = hz::locale_c_get();
+		hz::ScopedCLocale loc("", old_locale == "C");  // set system locale if the current one is C
+
+		struct lconv* lc = std::localeconv();
+		if (lc && lc->thousands_sep && *(lc->thousands_sep)) {
+			to_replace.push_back(lc->thousands_sep);
+		}
+	}  // the locale is restored here
+#endif
+
+	to_replace.push_back("bytes");
+	std::string s = hz::string_replace_array_copy(hz::string_trim_copy(str), to_replace, "");
 
 	uint64_t v = 0;
 	if (hz::string_is_numeric(s, v, false)) {
@@ -308,16 +355,13 @@ bool SmartctlParser::parse_section(const std::string& header, const std::string&
 // Parse the info section (without "===" header)
 bool SmartctlParser::parse_section_info(const std::string& body)
 {
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
 	this->set_data_section_info(body);
 
 	StorageProperty::section_t section = StorageProperty::section_info;
 
 	// split by lines.
 	// e.g. Device Model:     ST3500630AS
-	pcrecpp::RE re("^([^:\\n]+):[ \\t]*(.*)$", re_options);
+	pcrecpp::RE re = app_pcre_re("/^([^\\n]+): [ \\t]*(.*)$/miU");  // ungreedy
 
 	std::string name, value;
 	pcrecpp::StringPiece input(body);  // position tracker
@@ -357,11 +401,38 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 	}
 
 
-	if (app_pcre_match("/Model Family/mi", p.reported_name)
-			|| app_pcre_match("/Device Model/mi", p.reported_name)
-			|| app_pcre_match("/Serial Number/mi", p.reported_name)
-			|| app_pcre_match("/Firmware Version/mi", p.reported_name)
-			) {
+	if (app_pcre_match("/Model Family/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "model_family", "Model Family");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/Device Model/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "device_model", "Device Model");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/Serial Number/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "serial_number", "Serial Number");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/Firmware Version/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "firmware_version", "Firmware Version");
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/Sector Sizes/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "sector_sizes", "Sector Sizes");
+		p.value_type = StorageProperty::value_type_string;  // prints 2 values (phys/logical, if they're different)
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/Sector Size/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "sector_size", "Sector Size");
+		p.value_type = StorageProperty::value_type_string;  // prints a single value (if it's not 512)
+		p.value_string = p.reported_value;
+
+	} else if (app_pcre_match("/LU WWN Device Id/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "wwn_id", "World Wide Name");
 		p.value_type = StorageProperty::value_type_string;
 		p.value_string = p.reported_value;
 
@@ -376,6 +447,7 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 		p.value_string = p.reported_value;
 
 	} else if (app_pcre_match("/User Capacity/mi", p.reported_name)) {
+		p.set_name(p.reported_name, "capacity", "Capacity");
 		p.value_type = StorageProperty::value_type_integer;
 		uint64_t v = 0;
 		if ((p.readable_value = parse_byte_size(p.reported_value, v, true)).empty()) {
@@ -392,7 +464,7 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 			p.value_integer = v;
 
 	} else if (app_pcre_match("/Device is/mi", p.reported_name)) {
-		p.set_name(p.reported_name, "in_smartctl_db", "Is in smartctl database");
+		p.set_name(p.reported_name, "in_smartctl_db", "In Smartctl Database");
 		p.value_type = StorageProperty::value_type_bool;
 		p.value_bool = (!app_pcre_match("/Not in /mi", p.reported_value));
 
@@ -401,37 +473,40 @@ bool SmartctlParser::parse_section_info_property(StorageProperty& p)
 		// Don't put complete messages here - they change across smartctl versions.
 
 		if (app_pcre_match("/Available - device has/mi", p.reported_value)) {
-			p.set_name(p.reported_name, "smart_supported", "SMART supported");
+			p.set_name(p.reported_name, "smart_supported", "SMART Supported");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = true;
 
 		} else if (app_pcre_match("/Enabled/mi", p.reported_value)) {
-			p.set_name(p.reported_name, "smart_enabled", "SMART enabled");
+			p.set_name(p.reported_name, "smart_enabled", "SMART Enabled");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = true;
 
 		} else if (app_pcre_match("/Disabled/mi", p.reported_value)) {
-			p.set_name(p.reported_name, "smart_enabled", "SMART enabled");
+			p.set_name(p.reported_name, "smart_enabled", "SMART Enabled");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = false;
 
 		} else if (app_pcre_match("/Unavailable/mi", p.reported_value)) {
-			p.set_name(p.reported_name, "smart_supported", "SMART supported");
+			p.set_name(p.reported_name, "smart_supported", "SMART Supported");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = false;
 
 		// this should be last - when ambiguous state is detected, usually smartctl
 		// retries with other methods and prints one of the above.
 		} else if (app_pcre_match("/Ambiguous/mi", p.reported_value)) {
-			p.set_name(p.reported_name, "smart_supported", "SMART supported");
+			p.set_name(p.reported_name, "smart_supported", "SMART Supported");
 			p.value_type = StorageProperty::value_type_bool;
 			p.value_bool = true;  // let's be optimistic - just hope that it doesn't hurt.
 
 		}
 
 	} else {
-		debug_out_warn("app", DBG_FUNC_MSG << "Unknown attribute \"" << p.reported_name << "\"\n");
+		debug_out_warn("app", DBG_FUNC_MSG << "Unknown property \"" << p.reported_name << "\"\n");
 		// this is not an error, just unknown attribute. treat it as string.
+		// Don't highlight it with warning, it may just be a new smartctl feature.
+		p.value_type = StorageProperty::value_type_string;
+		p.value_string = p.reported_value;
 	}
 
 	return true;
@@ -464,7 +539,7 @@ bool SmartctlParser::parse_section_data(const std::string& body)
 	// merge "error log" parts. each part begins with a double-space or "Error nn".
 	for (unsigned int i = 0; i < split_subsections.size(); ++i) {
 		std::string sub = hz::string_trim_copy(split_subsections[i], "\t\n\r");  // don't trim space
-		if (pcrecpp::RE("^  ").PartialMatch(sub) || pcrecpp::RE("^Error [0-9]+").PartialMatch(sub)) {
+		if (app_pcre_re("^  ").PartialMatch(sub) || app_pcre_re("^Error [0-9]+").PartialMatch(sub)) {
 			if (!subsections.empty()) {
 				subsections.back() += "\n\n" + sub;  // append to previous part
 			} else {
@@ -525,20 +600,14 @@ bool SmartctlParser::parse_section_data_subsection_health(const std::string& sub
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_health;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
-
-	pcrecpp::RE re("^([^:\\n]+):[ \\t]*(.*)$", re_options);
-
 	std::string name, value;
-	if (re.PartialMatch(sub, &name, &value)) {
+	if (app_pcre_match("/^([^:\\n]+):[ \\t]*(.*)$/mi", sub, &name, &value)) {
 		hz::string_trim(name);
 		hz::string_trim(value);
 
 		// only one attribute in this section
 		if (app_pcre_match("/SMART overall-health self-assessment/mi", name)) {
-			pt.set_name(name, "overall_health", "Overall health self-assessment test");
+			pt.set_name(name, "overall_health", "Overall Health Self-Assessment Test");
 			pt.reported_value = value;
 			pt.value_type = StorageProperty::value_type_string;
 			pt.value_string = pt.reported_value;
@@ -563,16 +632,12 @@ bool SmartctlParser::parse_section_data_subsection_capabilities(const std::strin
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_capabilities;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
-
 	std::string sub = sub_initial;
 
 	// Fix some bugs in smartctl output (pre-5.39-final versions):
 	// There is a stale newline in "is in a Vendor Specific state\n.\n" and
 	// "is in a Reserved state\n.\n".
-// 	pcrecpp::RE("\\n\\.$", re_options).GlobalReplace(".", &sub);
+// 	app_pcre_replace("/\\n\\.$/mi", ".", &sub);
 	app_pcre_replace("/(is in a Vendor Specific state)\\n\\.$/mi", "\\1.", sub);
 	app_pcre_replace("/(is in a Reserved state)\\n\\.$/mi", "\\1.", sub);
 
@@ -608,8 +673,7 @@ bool SmartctlParser::parse_section_data_subsection_capabilities(const std::strin
 
 
 	// parse each block
-	pcrecpp::RE re("([^:]*):\\s*\\(([^)]+)\\)\\s*(.*)",
-			pcrecpp::RE_Options().set_multiline(true).set_dotall(true));
+	pcrecpp::RE re = app_pcre_re("/([^:]*):\\s*\\(([^)]+)\\)\\s*(.*)/ms");
 
 	bool cap_found = false;  // found at least one capability
 
@@ -676,7 +740,7 @@ bool SmartctlParser::parse_section_data_subsection_capabilities(const std::strin
 			p.value_type = StorageProperty::value_type_capability;
 
 			p.value_capability.reported_flag_value = numvalue_orig;
-			p.value_capability.flag_value = numvalue;  // full flag value
+			p.value_capability.flag_value = static_cast<int16_t>(numvalue);  // full flag value
 			p.value_capability.reported_strvalue = strvalue_orig;
 
 			// split capability lines into a vector. every flag sentence ends with "."
@@ -708,9 +772,6 @@ bool SmartctlParser::parse_section_data_subsection_capabilities(const std::strin
 // Check the capabilities for internal properties we can use.
 bool SmartctlParser::parse_section_data_internal_capabilities(StorageProperty& cap)
 {
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
 	// Some special capabilities we're interested in.
 
 	// Note: Smartctl gradually changed spelling Off-line to Offline in some messages.
@@ -721,29 +782,36 @@ bool SmartctlParser::parse_section_data_internal_capabilities(StorageProperty& c
 
 	// "was never started", "was completed without error", "is in progress",
 	// "was suspended by an interrupting command from host", etc...
-	pcrecpp::RE re_offline_status("^(Off-?line data collection) activity (?:is|was) (.*)$", re_options);
+	pcrecpp::RE re_offline_status = app_pcre_re("/^(Off-?line data collection) activity (?:is|was) (.*)$/mi");
 	// "Enabled", "Disabled". May not show up on older smartctl (< 5.1.10), so no way of knowing there.
-	pcrecpp::RE re_offline_enabled("^(Auto Off-?line Data Collection):[ \\t]*(.*)$", re_options);
-	pcrecpp::RE re_offline_immediate("^(SMART execute Off-?line immediate)$", re_options);
+	pcrecpp::RE re_offline_enabled = app_pcre_re("/^(Auto Off-?line Data Collection):[ \\t]*(.*)$/mi");
+	pcrecpp::RE re_offline_immediate = app_pcre_re("/^(SMART execute Off-?line immediate)$/mi");
 	// "No Auto Offline data collection support.", "Auto Offline data collection on/off support.".
-	pcrecpp::RE re_offline_auto("^(No |)(Auto Off-?line data collection (?:on\\/off )?support)$", re_options);
+	pcrecpp::RE re_offline_auto = app_pcre_re("/^(No |)(Auto Off-?line data collection (?:on\\/off )?support)$/mi");
 	// Same as above (smartctl <= 5.1-18). "No Automatic timer ON/OFF support."
-	pcrecpp::RE re_offline_auto2("^(No |)(Automatic timer ON\\/OFF support)$", re_options);
-	pcrecpp::RE re_offline_suspend("^(?:Suspend|Abort) (Off-?line collection upon new command)$", re_options);
-	pcrecpp::RE re_offline_surface("^(No |)(Off-?line surface scan supported)$", re_options);
-	pcrecpp::RE re_offline_time("^(Total time to complete Off-?line data collection)", re_options);  // match on name!
+	pcrecpp::RE re_offline_auto2 = app_pcre_re("/^(No |)(Automatic timer ON\\/OFF support)$/mi");
+	pcrecpp::RE re_offline_suspend = app_pcre_re("/^(?:Suspend|Abort) (Off-?line collection upon new command)$/mi");
+	pcrecpp::RE re_offline_surface = app_pcre_re("/^(No |)(Off-?line surface scan supported)$/mi");
 
-	pcrecpp::RE re_selftest_status("^Self-test execution status", re_options);  // match on name
-	pcrecpp::RE re_selftest_support("^(No |)(Self-test supported)$", re_options);
-	pcrecpp::RE re_conv_selftest_support("^(No |)(Conveyance Self-test supported)$", re_options);
-	pcrecpp::RE re_selective_selftest_support("^(No |)(Selective Self-test supported)$", re_options);
-	pcrecpp::RE re_selftest_short_time("^(Short self-test routine recommended polling time)", re_options);  // match on name!
-	pcrecpp::RE re_selftest_long_time("^(Extended self-test routine recommended polling time)", re_options);  // match on name!
-	pcrecpp::RE re_conv_selftest_time("^(Conveyance self-test routine recommended polling time)", re_options);  // match on name!
+	pcrecpp::RE re_selftest_support = app_pcre_re("/^(No |)(Self-test supported)$/mi");
+	pcrecpp::RE re_conv_selftest_support = app_pcre_re("/^(No |)(Conveyance Self-test supported)$/mi");
+	pcrecpp::RE re_selective_selftest_support = app_pcre_re("/^(No |)(Selective Self-test supported)$/mi");
 
-	pcrecpp::RE re_sct_status("^(SCT Status supported)$", re_options);
-	pcrecpp::RE re_sct_control("^(SCT Feature Control supported)$", re_options);  // means can change logging interval
-	pcrecpp::RE re_sct_data("^(SCT Data Table supported)$", re_options);
+	pcrecpp::RE re_sct_status = app_pcre_re("/^(SCT Status supported)$/mi");
+	pcrecpp::RE re_sct_control = app_pcre_re("/^(SCT Feature Control supported)$/mi");  // means can change logging interval
+	pcrecpp::RE re_sct_data = app_pcre_re("/^(SCT Data Table supported)$/mi");
+
+	// these are matched on name
+	pcrecpp::RE re_offline_status_group = app_pcre_re("/^(Off-?line data collection status)/mi");
+	pcrecpp::RE re_offline_time = app_pcre_re("/^(Total time to complete Off-?line data collection)/mi");
+	pcrecpp::RE re_offline_cap_group = app_pcre_re("/^(Off-?line data collection capabilities)/mi");
+	pcrecpp::RE re_smart_cap_group = app_pcre_re("/^(SMART capabilities)/mi");
+	pcrecpp::RE re_error_log_cap_group = app_pcre_re("/^(Error logging capability)/mi");
+	pcrecpp::RE re_sct_cap_group = app_pcre_re("/^(SCT capabilities)/mi");
+	pcrecpp::RE re_selftest_status = app_pcre_re("/^Self-test execution status/mi");
+	pcrecpp::RE re_selftest_short_time = app_pcre_re("/^(Short self-test routine recommended polling time)/mi");
+	pcrecpp::RE re_selftest_long_time = app_pcre_re("/^(Extended self-test routine recommended polling time)/mi");
+	pcrecpp::RE re_conv_selftest_time = app_pcre_re("/^(Conveyance self-test routine recommended polling time)/mi");
 
 	if (cap.section != StorageProperty::section_data || cap.subsection != StorageProperty::subsection_capabilities) {
 		debug_out_error("app", DBG_FUNC_MSG << "Non-capability property passed.\n");
@@ -751,9 +819,30 @@ bool SmartctlParser::parse_section_data_internal_capabilities(StorageProperty& c
 	}
 
 
+	// Name the capability groups for easy matching when setting descriptions
+	if (cap.value_type == StorageProperty::value_type_capability) {
+		if (re_offline_status_group.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "offline_status_group";
+
+		} else if (re_offline_cap_group.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "offline_cap_group";
+
+		} else if (re_smart_cap_group.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "smart_cap_group";
+
+		} else if (re_error_log_cap_group.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "error_log_cap_group";
+
+		} else if (re_sct_cap_group.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "sct_cap_group";
+
+		} else if (re_selftest_status.PartialMatch(cap.reported_name)) {
+			cap.generic_name = "last_selftest_cap_group";
+		}
+	}
 
 
-	// match on name:
+	// Last self-test status
 	if (re_selftest_status.PartialMatch(cap.reported_name)) {
 		// The last self-test status. break up into pieces.
 
@@ -852,6 +941,7 @@ bool SmartctlParser::parse_section_data_internal_capabilities(StorageProperty& c
 		return true;
 	}
 
+
 	// Extract subcapabilities from capability vectors and assign to "internal" section.
 	if (cap.value_type == StorageProperty::value_type_capability) {
 
@@ -955,32 +1045,53 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_attributes;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
-
 	// split to lines
 	std::vector<std::string> lines;
 	hz::string_split(sub, '\n', lines, true);
 
 	// Format notes:
-	// * Before 5.1-14, no UPDATED column was present.
+	// * Before 5.1-14, no UPDATED column was present in "old" format.
 	// * Most, but not all attribute names are with underscores. However, I encountered one
-	// named "Head flying hours". So, parse until we encounter the next column.
-	// * One WD drive had non-integer flags, something like "PO--C-", with several
-	// lines of their descriptions after the attributes block (each line started with spaces and |).
+	// named "Head flying hours" and there are slashes sometimes as well.
+	// So, parse until we encounter the next column.
+	// * "Old" format: One WD drive had non-integer flags, something like "PO--C-", with several
+	// lines of their descriptions after the attributes block (each line started with spaces and |)
+	// (seems to be a CVS version from 2006).
+	// * SSD drives may show "---" in value/worst/threshold fields.
 
+	// "old" format (used in -a):
+	// "ID# ATTRIBUTE_NAME          FLAG     VALUE WORST THRESH TYPE      UPDATED  WHEN_FAILED RAW_VALUE"
+	// "  1 Raw_Read_Error_Rate     0x000f   115   099   006    Pre-fail  Always       -       90981479"
+	// " 12 Power_Cycle_Count       0x0000   ---   ---   ---    Old_age   Offline      -       167"
+
+	// "brief" format (used in -x):
+	// "ID# ATTRIBUTE_NAME          FLAGS    VALUE WORST THRESH FAIL RAW_VALUE"
+	// "  1 Raw_Read_Error_Rate     POSR--   118   099   006    -    174772734"
+
+	enum {
+		FormatStyleOld,
+		FormatStyleNoUpdated,  // old format without UPDATED column
+		FormatStyleBrief
+	};
 
 	bool attr_found = false;  // at least one attribute was found
-	bool attr_format_with_updated = false;  // UPDATED column present
+	int attr_format_style = FormatStyleOld;
 
-	pcrecpp::RE re_up("[ \\t]*([0-9]+) ([^\\t\\n]+)[ \\t]+((?:0x[a-fA-F0-9]+)|(?:[A-Z-]{2,}))[ \\t]+([0-9]+)[ \\t]+([0-9]+)[ \\t]+([0-9]+)[ \\t]+"
-			"([^ \\t\\n]+)[ \\t]+([^ \\t\\n]+)[ \\t]+([^ \\t\\n]+)[ \\t]+(.+)[ \\t]*", re_options);
+	std::string space_re = "[ \\t]+";
 
-	pcrecpp::RE re_noup("[ \\t]*([0-9]+) ([^\\t\\n]+)[ \\t]+((?:0x[a-fA-F0-9]+)|(?:[A-Z-]{2,}))[ \\t]+([0-9]+)[ \\t]+([0-9]+)[ \\t]+([0-9]+)[ \\t]+"
-			"([^ \\t\\n]+)[ \\t]+([^ \\t\\n]+)[ \\t]+(.+)[ \\t]*", re_options);
+	std::string flag_re = "((?:0x[a-fA-F0-9]+)|(?:[A-Z+-]{2,}))";
+	std::string base_re = "[ \\t]*([0-9]+) ([^\\t\\n]+)" + space_re + flag_re + space_re  // name / flag
+			+ "([0-9-]+)" + space_re + "([0-9-]+)" + space_re + "([0-9-]+)" + space_re;  // value / worst / threshold
+	std::string type_re = "([^ \\t\\n]+)" + space_re;
+	std::string updated_re = "([^ \\t\\n]+)" + space_re;
+	std::string failed_re = "([^ \\t\\n]+)" + space_re;
+	std::string raw_re = "(.+)[ \\t]*";
 
-	pcrecpp::RE re_flag_descr("^[\\t ]+\\|", re_options);
+	pcrecpp::RE re_old_up = app_pcre_re("/" + base_re + type_re + updated_re + failed_re + raw_re + "/mi");
+	pcrecpp::RE re_old_noup = app_pcre_re("/" + base_re + type_re + failed_re + raw_re + "/mi");
+	pcrecpp::RE re_brief = app_pcre_re("/" + base_re + failed_re + raw_re + "/mi");
+
+	pcrecpp::RE re_flag_descr = app_pcre_re("/^[\\t ]+\\|/mi");
 
 
 	for(unsigned int i = 0; i < lines.size(); ++i) {
@@ -991,7 +1102,12 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 			continue;
 
 		if (app_pcre_match("/ATTRIBUTE_NAME/mi", line)) {
-			attr_format_with_updated = (app_pcre_match("/UPDATED/mi", line));
+			// detect format type
+			if (!app_pcre_match("/WHEN_FAILED/mi", line)) {
+				attr_format_style = FormatStyleBrief;
+			} else if (!app_pcre_match("/UPDATED/mi", line)) {
+				attr_format_style = FormatStyleNoUpdated;
+			}
 			continue;  // we don't need this line
 		}
 
@@ -1000,7 +1116,7 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 		}
 
 		if (app_pcre_match("/Data Structure revision number/mi", line)) {
-			pcrecpp::RE re("^([^:\\n]+):[ \\t]*(.*)$", re_options);
+			pcrecpp::RE re = app_pcre_re("/^([^:\\n]+):[ \\t]*(.*)$/mi");
 			std::string name, value;
 			if (re.PartialMatch(line, &name, &value)) {
 				hz::string_trim(name);
@@ -1009,7 +1125,7 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 				hz::string_is_numeric(value, value_num, false);
 
 				StorageProperty p(pt);
-				p.set_name(name);
+				p.set_name(name, "data_structure_version");
 				p.reported_value = value;
 				p.value_type = StorageProperty::value_type_integer;
 				p.value_integer = value_num;
@@ -1026,15 +1142,22 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 
 			bool matched = true;
 
-			if (attr_format_with_updated) {
-				if (!re_up.FullMatch(line, &id, &name, &flag, &value, &worst, &threshold, &attr_type,
+			if (attr_format_style == FormatStyleOld) {
+				if (!re_old_up.FullMatch(line, &id, &name, &flag, &value, &worst, &threshold, &attr_type,
 						&update_type, &when_failed, &raw_value)) {
 					matched = false;
 					debug_out_warn("app", DBG_FUNC_MSG << "Cannot parse attribute line.\n");
 				}
 
-			} else {  // no UPDATED column
-				if (!re_noup.FullMatch(line, &id, &name, &flag, &value, &worst, &threshold, &attr_type,
+			} else if (attr_format_style == FormatStyleNoUpdated) {
+				if (!re_old_noup.FullMatch(line, &id, &name, &flag, &value, &worst, &threshold, &attr_type,
+						&when_failed, &raw_value)) {
+					matched = false;
+					debug_out_warn("app", DBG_FUNC_MSG << "Cannot parse attribute line.\n");
+				}
+
+			} else if (attr_format_style == FormatStyleBrief) {
+				if (!re_brief.FullMatch(line, &id, &name, &flag, &value, &worst, &threshold,
 						&when_failed, &raw_value)) {
 					matched = false;
 					debug_out_warn("app", DBG_FUNC_MSG << "Cannot parse attribute line.\n");
@@ -1052,23 +1175,49 @@ bool SmartctlParser::parse_section_data_subsection_attributes(const std::string&
 			StorageAttribute a;
 			hz::string_is_numeric(hz::string_trim_copy(id), a.id, true, 10);
 			a.flag = hz::string_trim_copy(flag);
-			hz::string_is_numeric(hz::string_trim_copy(value), a.value, true, 10);
-			hz::string_is_numeric(hz::string_trim_copy(worst), a.worst, true, 10);
-			hz::string_is_numeric(hz::string_trim_copy(threshold), a.threshold, true, 10);
+			uint8_t norm_value = 0, worst_value = 0, threshold_value = 0;
 
-			a.attr_type = (attr_type == "Pre-fail" ? StorageAttribute::attr_type_prefail
-					: (attr_type == "Old_age" ? StorageAttribute::attr_type_oldage : StorageAttribute::attr_type_unknown));
+			if (hz::string_is_numeric(hz::string_trim_copy(value), norm_value, true, 10)) {
+				a.value = norm_value;
+			}
+			if (hz::string_is_numeric(hz::string_trim_copy(worst), worst_value, true, 10)) {
+				a.worst = worst_value;
+			}
+			if (hz::string_is_numeric(hz::string_trim_copy(threshold), threshold_value, true, 10)) {
+				a.threshold = threshold_value;
+			}
 
-			a.update_type = (update_type == "Always" ? StorageAttribute::update_type_always
-					: (update_type == "Offline" ? StorageAttribute::update_type_offline : StorageAttribute::update_type_unknown));
+			if (attr_format_style == FormatStyleBrief) {
+				a.attr_type = app_pcre_match("/P/", a.flag) ? StorageAttribute::attr_type_prefail : StorageAttribute::attr_type_oldage;
+			} else {
+				if (attr_type == "Pre-fail") {
+					a.attr_type = StorageAttribute::attr_type_prefail;
+				} else if (attr_type == "Old_age") {
+					a.attr_type = StorageAttribute::attr_type_oldage;
+				} else {
+					a.attr_type = StorageAttribute::attr_type_unknown;
+				}
+			}
+
+			if (attr_format_style == FormatStyleBrief) {
+				a.update_type = app_pcre_match("/O/", a.flag) ? StorageAttribute::update_type_always : StorageAttribute::update_type_offline;
+			} else {
+				if (update_type == "Always") {
+					a.update_type = StorageAttribute::update_type_always;
+				} else if (update_type == "Offline") {
+					a.update_type = StorageAttribute::update_type_offline;
+				} else {
+					a.update_type = StorageAttribute::update_type_unknown;
+				}
+			}
 
 			a.when_failed = StorageAttribute::fail_time_unknown;
 			hz::string_trim(when_failed);
 			if (when_failed == "-") {
 				a.when_failed = StorageAttribute::fail_time_none;
-			} else if (when_failed == "In_the_past") {
+			} else if (when_failed == "In_the_past" || when_failed == "Past") {  // the second one if from brief format
 				a.when_failed = StorageAttribute::fail_time_past;
-			} else if (when_failed == "FAILING_NOW") {
+			} else if (when_failed == "FAILING_NOW" || when_failed == "NOW") {  // the second one if from brief format
 				a.when_failed = StorageAttribute::fail_time_now;
 			}
 
@@ -1104,10 +1253,6 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_error_log;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
-
 	// Note: The format of this section was changed somewhere between 5.0-x and 5.30.
 	// The old format is doesn't really give any useful info, and whatever's left is somewhat
 	// parsable by this parser. Can't really improve that.
@@ -1115,9 +1260,9 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 
 	bool data_found = false;
 
-	// error log version
+	// Error log version
 	{
-		pcrecpp::RE re("^(SMART Error Log Version):[ \\t]*(.*)$", re_options);
+		pcrecpp::RE re = app_pcre_re("/^(SMART Error Log Version):[ \\t]*(.*)$/mi");
 
 		std::string name, value;
 		if (re.PartialMatch(sub, &name, &value)) {
@@ -1138,11 +1283,24 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 		}
 	}
 
-	// error log count
+	// Error log support
+	{
+		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Error Logging)$/mi");
+
+		if (re.PartialMatch(sub)) {
+			StorageProperty p(pt);
+			p.set_name("error_log_unsupported");
+			p.readable_name = "Warning";
+			p.readable_value = "Device does not support error logging";
+			add_property(p);
+		}
+	}
+
+	// Error log enty count
 	{
 		// note: these represent the same information
-		pcrecpp::RE re1("^ATA Error Count:[ \\t]*([0-9]+)", re_options);
-		pcrecpp::RE re2("^No Errors Logged$", re_options);
+		pcrecpp::RE re1 = app_pcre_re("/^ATA Error Count:[ \\t]*([0-9]+)/mi");
+		pcrecpp::RE re2 = app_pcre_re("/^No Errors Logged$/mi");
 
 		std::string value;
 		if (re1.PartialMatch(sub, &value) || re2.PartialMatch(sub)) {
@@ -1167,11 +1325,14 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 	// individual errors
 	{
 		// split by blocks
-		pcrecpp::RE re_block("^((Error[ \\t]*([0-9]+))[ \\t]*occurred at disk power-on lifetime:[ \\t]*([0-9]+) hours.*(?:\\n(?:  |\\n  ).*)*)", re_options);
+		pcrecpp::RE re_block = app_pcre_re("/^((Error[ \\t]*([0-9]+))[ \\t]*occurred at disk power-on lifetime:[ \\t]*([0-9]+) hours.*(?:\\n(?:  |\\n  ).*)*)/mi");
 
-		// for "in an unknown state" - remove first two words.
-		pcrecpp::RE re_state("occurred, the device was[ \\t]*(?: in)?(?: an?)?[ \\t]+([^.\\n]*)\\.?", re_options);
-		pcrecpp::RE re_type("[ \\t]+Error:[ \\t]*([^ \\t\\n]+)[ \\t]*(.*)$", re_options);
+		// "  When the command that caused the error occurred, the device was active or idle."
+		// Note: For "in an unknown state" - remove first two words.
+		pcrecpp::RE re_state = app_pcre_re("/occurred, the device was[ \\t]*(?: in)?(?: an?)?[ \\t]+([^.\\n]*)\\.?/mi");
+		// "  84 51 2c 71 cd 3f e6  Error: ICRC, ABRT 44 sectors at LBA = 0x063fcd71 = 104844657"
+		// "  40 51 00 f5 41 61 e0  Error: UNC at LBA = 0x006141f5 = 6373877"
+		pcrecpp::RE re_type = app_pcre_re("/[ \\t]+Error:[ \\t]*([ ,a-z]+)[ \\t]+((?:[0-9]+|at )[ \\t]*.*)$/mi");
 
 		std::string block, name, value_num, value_time;
 		pcrecpp::StringPiece input(sub);  // position tracker
@@ -1182,9 +1343,9 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 			hz::string_trim(value_time);
 			// debug_out_dump("app", "\nBLOCK -------------------------------\n" << block);
 
-			std::string state, etype, emore;
+			std::string state, etypes_str, emore;
 			re_state.PartialMatch(block, &state);
-			re_type.PartialMatch(block, &etype, &emore);
+			re_type.PartialMatch(block, &etypes_str, &emore);
 
 			StorageProperty p(pt);
 			p.set_name(hz::string_trim_copy(name));  // "Error 6"
@@ -1194,8 +1355,14 @@ bool SmartctlParser::parse_section_data_subsection_error_log(const std::string&
 			hz::string_is_numeric(value_num, p.value_error_block.error_num, false);
 			hz::string_is_numeric(value_time, p.value_error_block.lifetime_hours, false);
 
+			std::vector<std::string> etypes;
+			hz::string_split(etypes_str, ",", etypes, true);
+			for (std::vector<std::string>::iterator iter = etypes.begin(); iter != etypes.end(); ++iter) {
+				hz::string_trim(*iter);
+			}
+
 			p.value_error_block.device_state = hz::string_trim_copy(state);
-			p.value_error_block.reported_type = hz::string_trim_copy(etype);
+			p.value_error_block.reported_types = etypes;
 			p.value_error_block.type_more_info = hz::string_trim_copy(emore);
 
 			add_property(p);
@@ -1234,9 +1401,6 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_selftest_log;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
 	// Self-test log contains:
 	// * structure revision number
 	// * a list of current / previous tests performed, with each having:
@@ -1249,7 +1413,7 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 
 	bool data_found = false;  // true if something was found.
 
-	// the whole subsection
+	// The whole subsection
 	{
 		StorageProperty p(pt);
 		p.set_name("SMART Self-test log", "selftest_log");
@@ -1262,12 +1426,25 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 	}
 
 
-	// self-test log version
+	// Self-test log support
+	{
+		pcrecpp::RE re = app_pcre_re("/^(Warning: device does not support Self Test Logging)$/mi");
+
+		if (re.PartialMatch(sub)) {
+			StorageProperty p(pt);
+			p.set_name("selftest_log_unsupported");
+			p.readable_name = "Warning";
+			p.readable_value = "Device does not support self-test logging";
+			add_property(p);
+		}
+	}
+
+	// Self-test log version
 	{
 		// newer smartctl (since smartctl 5.1-16)
-		pcrecpp::RE re1("(SMART Self-test log structure[^\\n0-9]*)([^ \\n]+)[ \\t]*$", re_options);
+		pcrecpp::RE re1 = app_pcre_re("/(SMART Self-test log structure[^\\n0-9]*)([^ \\n]+)[ \\t]*$/mi");
 		// older smartctl
-		pcrecpp::RE re2("(SMART Self-test log, version number[^\\n0-9]*)([^ \\n]+)[ \\t]*$", re_options);
+		pcrecpp::RE re2 = app_pcre_re("/(SMART Self-test log, version number[^\\n0-9]*)([^ \\n]+)[ \\t]*$/mi");
 
 		std::string name, value;
 		if (re1.PartialMatch(sub, &name, &value) || re2.PartialMatch(sub, &name, &value)) {
@@ -1295,9 +1472,9 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 
 	// individual entries
 	{
-		// split by columns (minimum 2 spaces between them)
-		// num, type, status, remaining, hours, lba
-		pcrecpp::RE re("^(#[ \\t]*([0-9]+)[ \\t]*(\\S.*)  [ \\t]*(\\S.*)  [ \\t]*([0-9]+%)  [ \\t]*([0-9]+)  [ \\t]*(\\S.*))$", re_options);
+		// split by columns.
+		// num, type, status, remaining, hours, lba (optional).
+		pcrecpp::RE re = app_pcre_re("/^(#[ \\t]*([0-9]+)[ \\t]+(\\S+(?: \\S+)*)  [ \\t]*(\\S.*) [ \\t]*([0-9]+%)  [ \\t]*([0-9]+)[ \\t]*((?:  [ \\t]*\\S.*)?))$/mi");
 
 		std::string line, num, type, status_str, remaining, hours, lba;
 		pcrecpp::StringPiece input(sub);  // position tracker
@@ -1316,6 +1493,9 @@ bool SmartctlParser::parse_section_data_subsection_selftest_log(const std::strin
 
 			p.value_selftest_entry.type = hz::string_trim_copy(type);
 			p.value_selftest_entry.lba_of_first_error = hz::string_trim_copy(lba);
+			// old smartctls didn't print anything for lba if none, but newer ones print "-". normalize.
+			if (p.value_selftest_entry.lba_of_first_error == "")
+				p.value_selftest_entry.lba_of_first_error = "-";
 
 			hz::string_trim(status_str);
 			StorageSelftestEntry::status_t status = StorageSelftestEntry::status_unknown;
@@ -1384,10 +1564,6 @@ bool SmartctlParser::parse_section_data_subsection_selective_selftest_log(const
 	pt.section = StorageProperty::section_data;
 	pt.subsection = StorageProperty::subsection_selective_selftest_log;
 
-	pcrecpp::RE_Options re_options;
-	re_options.set_caseless(true).set_multiline(true);
-
-
 	// Selective self-test log contains:
 	// * 5 (maybe less/more?) test entries, each with
 	// 		span / min_lba / max_lba / current_test_status.
