@@ -1,7 +1,7 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_whatever.txt
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+ License: See LICENSE_unlicense.txt
 ***************************************************************************/
 
 #include <iostream>
@@ -24,14 +24,15 @@ int main()
 // 	sd.add_match_patterns(match_patterns);
 	sd.add_blacklist_patterns(blacklist_patterns);
 
-	std::string error_msg = sd.detect_and_fetch_basic_data(drives);
+	ExecutorFactoryRefPtr ex_factory(new ExecutorFactory(false));
+	std::string error_msg = sd.detect_and_fetch_basic_data(drives, ex_factory);
 	if (!error_msg.empty()) {
 		std::cerr << error_msg << "\n";
 
 	} else {
 		for (unsigned int i = 0; i < drives.size(); ++i) {
-			std::cerr << drives[i]->get_device() <<
-					" (" << StorageDevice::get_type_readable_name(drives[i]->get_type()) << ")\n";
+			std::cerr << drives[i]->get_device_with_type() <<
+					" (" << StorageDevice::get_type_readable_name(drives[i]->get_detected_type()) << ")\n";
 		}
 	}
 
