@@ -1,11 +1,9 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
-#include <glibmm.h>  // Glib::shell_quote()
-
 #include "rconfig/rconfig_mini.h"
 #include "hz/string_algo.h"  // string_trim_copy, string_any_to_unix_copy
 #include "hz/fs_path.h"  // FsPath
@@ -20,8 +18,134 @@
 
 
 
-// Fetch and parse basic info
-// note: this will clear the non-basic properties!
+
+std::string StorageDevice::get_type_readable_name(StorageDevice::detected_type_t type)
+{
+    switch (type) {
+    case detected_type_unknown:
+        return "unknown";
+    case detected_type_invalid:
+        return "invalid";
+    case detected_type_cddvd:
+        return "cd/dvd";
+    }
+    return "[internal_error]";
+}
+
+
+
+std::string StorageDevice::get_status_name(StorageDevice::status_t status, bool use_yesno)
+{
+    switch (status) {
+    case status_enabled:
+        return (use_yesno ? "Yes" : "Enabled");
+    case status_disabled:
+        return (use_yesno ? "No" : "Disabled");
+    case status_unsupported:
+        return "Unsupported";
+    case status_unknown:
+        return "Unknown";
+    };
+    return "[internal_error]";
+}
+
+
+
+StorageDevice::StorageDevice(const string& dev_or_vfile, bool is_virtual)
+{
+	detected_type_ = detected_type_unknown;
+	// force_type_ = false;
+	is_virtual_ = is_virtual;
+	is_manually_added_ = false;
+	fully_parsed_ = false;
+	test_is_active_ = false;
+
+	if (is_virtual) {
+		virtual_file_ = dev_or_vfile;
+	} else {
+		device_ = dev_or_vfile;
+	}
+}
+
+
+
+StorageDevice::StorageDevice(const string& dev, const string& type_arg)
+{
+	detected_type_ = detected_type_unknown;
+	// force_type_ = false;
+	is_virtual_ = false;
+	is_manually_added_ = false;
+	fully_parsed_ = false;
+	test_is_active_ = false;
+
+	device_ = dev;
+	type_arg_ = type_arg;
+}
+
+
+
+StorageDevice::StorageDevice(const StorageDevice& other)
+{
+    *this = other;
+}
+
+
+
+StorageDevice& StorageDevice::operator=(const StorageDevice& other)
+{
+    info_output_ = other.info_output_;
+    full_output_ = other.full_output_;
+
+    device_ = other.device_;
+	type_arg_ = other.type_arg_;
+	extra_args_ = other.extra_args_;
+
+    // force_type_ = other.force_type_;
+    is_virtual_ = other.is_virtual_;
+    virtual_file_ = other.virtual_file_;
+    is_manually_added_ = other.is_manually_added_;
+
+    fully_parsed_ = other.fully_parsed_;
+    test_is_active_ = other.test_is_active_;
+
+    detected_type_ = other.detected_type_;
+    smart_supported_ = other.smart_supported_;
+    smart_enabled_ = other.smart_enabled_;
+    aodc_status_ = other.aodc_status_;
+    model_name_ = other.model_name_;
+    family_name_ = other.family_name_;
+    size_ = other.size_;
+    health_property_ = other.health_property_;
+
+    properties_ = other.properties_;
+
+    return *this;
+}
+
+
+
+void StorageDevice::clear_fetched(bool including_outputs) {
+    if (including_outputs) {
+        info_output_.clear();
+        full_output_.clear();
+    }
+
+    fully_parsed_ = false;
+    test_is_active_ = false;  // not sure
+
+    smart_supported_.reset();
+    smart_enabled_.reset();
+    model_name_.reset();
+    aodc_status_.reset();
+    family_name_.reset();
+    size_.reset();
+    health_property_.reset();
+
+    properties_.clear();
+}
+
+
+
 std::string StorageDevice::fetch_basic_data_and_parse(hz::intrusive_ptr<CmdexSync> smartctl_ex)
 {
 	if (this->test_is_active_)
@@ -29,28 +153,29 @@ std::string StorageDevice::fetch_basic_data_and_parse(hz::intrusive_ptr<CmdexSyn
 
 	this->clear_fetched();  // clear everything fetched before, including outputs
 
-	std::string output;
 	// We don't use "--all" - it may cause really screwed up the output (tests, etc...).
 	// This looks just like "--info" only on non-smart devices.
 	// --info --health --capabilities
-	std::string error_msg = execute_smartctl("-i -H -c", smartctl_ex, output, true);  // set type to invalid if needed
+	std::string error_msg = execute_device_smartctl("-i -H -c", smartctl_ex, this->info_output_, true);  // set type to invalid if needed
 
 	// Smartctl 5.39 cvs/svn version defaults to usb type on at least linux and windows.
 	// This means that the old SCSI identify command isn't executed by default,
 	// and there is no information about the device manufacturer/etc... in the output.
 	// We detect this and set the device type to scsi to at least have _some_ info.
-	if (get_type() == type_invalid) {
+	if (get_detected_type() == detected_type_invalid && get_type_argument().empty()) {
 		debug_out_info("app", "The device seems to be of different type than auto-detected, trying again with scsi.\n");
-		this->set_type(type_scsi);
+		this->set_type_argument("scsi");
 		return this->fetch_basic_data_and_parse(smartctl_ex);  // try again with scsi
 	}
 
 	// Since the type error leads to "command line didn't parse" error here,
 	// we do this after the scsi stuff.
-	if (!error_msg.empty())
+	if (!error_msg.empty()) {
+		// Still try to parse something. For some reason, running smartctl on usb flash drive
+		// under winxp returns "command line didn't parse", while actually printing its name.
+		this->parse_basic_data(false, true);
 		return error_msg;
-
-	this->info_output_ = output;
+	}
 
 	// Set some properties too - they are needed for e.g. AODC status, etc...
 	return this->parse_basic_data(true);
@@ -58,8 +183,6 @@ std::string StorageDevice::fetch_basic_data_and_parse(hz::intrusive_ptr<CmdexSyn
 
 
 
-
-// note: this will clear the non-basic properties!
 std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_signal)
 {
 	this->clear_fetched(false);  // clear everything fetched before, except outputs
@@ -75,13 +198,7 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 
 	// detect type. note: we can't distinguish between sata and scsi (on linux, for -d ata switch).
 	if (app_pcre_match("/this device: CD\\/DVD/mi", info_output_)) {
-		this->set_type(type_cddvd);
-
-// 	} else {
-// 		std::string dev_base = get_device_base();
-// 		if (!dev_base.empty() && dev_base[0] == 'h') {  // e.g. hda
-// 			this->set_type(type_pata);
-// 		}
+		this->set_detected_type(detected_type_cddvd);
 	}
 
 	// Note: We don't use SmartctlParser here, because this information
@@ -92,7 +209,7 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 	// Don't put complete messages here - they change across smartctl versions.
 	if (app_pcre_match("/^SMART support is:[ \\t]*Unavailable/mi", info_output_)  // cdroms output this
 			|| app_pcre_match("/Device does not support SMART/mi", info_output_)  // usb flash drives, non-smart hds
-			|| app_pcre_match("/Device Read Identity Failed \\(not an ATA\\/ATAPI device\\)/mi", info_output_)) {  // solaris scsi, unsupported by smartctl
+			|| app_pcre_match("/Device Read Identity Failed/mi", info_output_)) {  // solaris scsi, unsupported by smartctl (maybe others?)
 		smart_supported_ = false;
 		smart_enabled_ = false;
 
@@ -112,16 +229,21 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 	if (app_pcre_match("/^Device Model:[ \\t]*(.*)$/mi", info_output_, &model)) {  // HD's and cdroms
 		model_name_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(model), ' ');
 
-	} else if (app_pcre_match("/^Device:[ \\t]*(.*)$/mi", info_output_, &model)) {  // usb flash drives
+	} else if (app_pcre_match("/^(?:Device|Product):[ \\t]*(.*)$/mi", info_output_, &model)) {  // usb flash drives
 		model_name_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(model), ' ');
 	}
 
 
-	std::string family;  // this is from smartctl's database I think
+	std::string family;  // this is from smartctl's database
 	if (app_pcre_match("/^Model Family:[ \\t]*(.*)$/mi", info_output_, &family)) {
 		family_name_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(family), ' ');
 	}
 
+	std::string serial;
+	if (app_pcre_match("/^Serial Number:[ \\t]*(.*)$/mi", info_output_, &serial)) {
+		serial_number_ = hz::string_remove_adjacent_duplicates_copy(hz::string_trim_copy(serial), ' ');
+	}
+
 
 	// Note: this property is present since 5.33.
 	std::string size;
@@ -149,7 +271,6 @@ std::string StorageDevice::parse_basic_data(bool do_set_properties, bool emit_si
 
 
 
-// fetch complete data (basic too), parse it.
 std::string StorageDevice::fetch_data_and_parse(hz::intrusive_ptr<CmdexSync> smartctl_ex)
 {
 	if (this->test_is_active_)
@@ -162,19 +283,19 @@ std::string StorageDevice::fetch_data_and_parse(hz::intrusive_ptr<CmdexSync> sma
 
 	// instead of -a, we use all the individual options -a encompasses, so that
 	// an addition to default -a output won't affect us.
-	if (this->get_type() == type_scsi) {
+	if (this->get_type_argument() == "scsi") {  // not sure about correctness... FIXME probably fails with RAID/scsi
 		// This doesn't do much yet, but just in case...
 		// SCSI equivalent of -a:
-		error_msg = execute_smartctl("-H -i -A -l error -l selftest", smartctl_ex, output);
+		error_msg = execute_device_smartctl("-H -i -A -l error -l selftest", smartctl_ex, output);
 	} else {
 		// ATA equivalent of -a:
-		error_msg = execute_smartctl("-H -i -c -A -l error -l selftest -l selective",
+		error_msg = execute_device_smartctl("-H -i -c -A -l error -l selftest -l selective",
 				smartctl_ex, output, true);  // set type to invalid if needed
 	}
 	// See notes above (in fetch_basic_data_and_parse()).
-	if (get_type() == type_invalid) {
+	if (get_detected_type() == detected_type_invalid && get_type_argument().empty()) {
 		debug_out_info("app", "The device seems to be of different type than auto-detected, trying again with scsi.\n");
-		this->set_type(type_scsi);
+		this->set_type_argument("scsi");
 		return this->fetch_data_and_parse(smartctl_ex);  // try again with scsi
 	}
 
@@ -189,9 +310,6 @@ std::string StorageDevice::fetch_data_and_parse(hz::intrusive_ptr<CmdexSync> sma
 
 
 
-
-// parses full info. if failed, tries to parse it as basic info.
-// returns an error message on error.
 std::string StorageDevice::parse_data()
 {
 	this->clear_fetched(false);  // clear everything fetched before, except outputs
@@ -234,7 +352,13 @@ std::string StorageDevice::parse_data()
 
 
 
-// returns error message on error, empty string on success
+bool StorageDevice::get_fully_parsed() const
+{
+	return fully_parsed_;
+}
+
+
+
 std::string StorageDevice::set_smart_enabled(bool b, hz::intrusive_ptr<CmdexSync> smartctl_ex)
 {
 	if (this->test_is_active_)
@@ -257,7 +381,7 @@ A mandatory SMART command failed: exiting. To continue, add one or more '-T perm
 
 	std::string output;
 	// --smart=on --saveauto=on, --smart=off
-	std::string error_msg = execute_smartctl((b ? "-s on -S on" : "-s off"), smartctl_ex, output);
+	std::string error_msg = execute_device_smartctl((b ? "-s on -S on" : "-s off"), smartctl_ex, output);
 	if (!error_msg.empty())
 		return error_msg;
 
@@ -274,7 +398,6 @@ A mandatory SMART command failed: exiting. To continue, add one or more '-T perm
 
 
 
-// returns error message on error, empty string on success
 std::string StorageDevice::set_aodc_enabled(bool b, hz::intrusive_ptr<CmdexSync> smartctl_ex)
 {
 	if (this->test_is_active_)
@@ -293,7 +416,7 @@ A mandatory SMART command failed: exiting. To continue, add one or more '-T perm
 */
 	std::string output;
 	// --offlineauto=on, --offlineauto=off
-	std::string error_msg = execute_smartctl((b ? "-o on" : "-o off"), smartctl_ex, output);
+	std::string error_msg = execute_device_smartctl((b ? "-o on" : "-o off"), smartctl_ex, output);
 	if (!error_msg.empty())
 		return error_msg;
 
@@ -309,16 +432,6 @@ A mandatory SMART command failed: exiting. To continue, add one or more '-T perm
 
 
 
-
-// get only the filename portion
-std::string StorageDevice::get_virtual_filename() const
-{
-	return (is_virtual_ ? hz::FsPath(virtual_file_).get_basename() : std::string());
-}
-
-
-
-
 StorageDevice::status_t StorageDevice::get_smart_status() const
 {
 	status_t status = status_unsupported;
@@ -397,7 +510,6 @@ StorageDevice::status_t StorageDevice::get_aodc_status() const
 
 
 
-// returns empty string on error, format size string on success.
 std::string StorageDevice::get_device_size_str() const
 {
 	return (size_.defined() ? size_.value() : "");
@@ -420,17 +532,235 @@ StorageProperty StorageDevice::get_health_property() const
 
 
 
-std::string StorageDevice::get_save_filename() const
+string StorageDevice::get_device() const
 {
-	std::string filename = this->get_model_name();  // may be empty
-	filename += hz::format_date("-%Y-%m-%d", false);
-	if (!filename.empty())
-		filename = hz::filename_make_safe(filename) + ".txt";
-	return filename;
+	return device_;
 }
 
 
 
+string StorageDevice::get_device_base() const
+{
+	if (is_virtual_)
+		return "";
+
+	std::string::size_type pos = device_.rfind('/');  // find basename
+	if (pos == std::string::npos)
+		return device_;  // fall back
+	return device_.substr(pos+1, std::string::npos);
+}
+
+
+
+string StorageDevice::get_device_pretty() const
+{
+	std::string ret;
+	if (this->get_is_virtual()) {
+		ret = "Virtual";
+		std::string vf = this->get_virtual_filename();
+		ret += (" (" + (vf.empty() ? "[empty]" : vf) + ")");
+	} else {
+		ret = this->get_device_with_type();
+	}
+	return ret;
+}
+
+
+
+string StorageDevice::get_device_with_type() const
+{
+	std::string device = get_device();
+	if (!get_type_argument().empty()) {
+		device += " (" + get_type_argument() + ")";
+	}
+	return device;
+}
+
+
+
+void StorageDevice::set_detected_type(StorageDevice::detected_type_t t)
+{
+	detected_type_ = t;
+}
+
+
+
+StorageDevice::detected_type_t StorageDevice::get_detected_type() const
+{
+	return detected_type_;
+}
+
+
+
+void StorageDevice::set_type_argument(const string& arg)
+{
+	type_arg_ = arg;
+}
+
+
+
+std::string StorageDevice::get_type_argument() const
+{
+	return type_arg_;
+}
+
+
+
+void StorageDevice::set_extra_arguments(const string& args)
+{
+	extra_args_ = args;
+}
+
+
+
+std::string StorageDevice::get_extra_arguments() const
+{
+	return extra_args_;
+}
+
+
+
+bool StorageDevice::get_is_virtual() const
+{
+	return is_virtual_;
+}
+
+
+
+string StorageDevice::get_virtual_file() const
+{
+	return (is_virtual_ ? virtual_file_ : std::string());
+}
+
+
+
+std::string StorageDevice::get_virtual_filename() const
+{
+	return (is_virtual_ ? hz::FsPath(virtual_file_).get_basename() : std::string());
+}
+
+
+
+const SmartctlParser::prop_list_t& StorageDevice::get_properties() const
+{
+	return properties_;
+}
+
+
+
+StorageProperty StorageDevice::lookup_property(const string& generic_name, StorageProperty::section_t section, StorageProperty::subsection_t subsection) const
+{
+	for (SmartctlParser::prop_list_t::const_iterator iter = properties_.begin(); iter != properties_.end(); ++iter) {
+		if (section != StorageProperty::section_unknown && iter->section != section)
+			continue;
+		if (subsection != StorageProperty::subsection_unknown && iter->subsection != subsection)
+			continue;
+
+		if (iter->generic_name == generic_name)
+			return *iter;
+	}
+	return StorageProperty();  // check with .empty()
+}
+
+
+
+string StorageDevice::get_model_name() const
+{
+	return (model_name_.defined() ? model_name_.value() : "");
+}
+
+
+
+string StorageDevice::get_family_name() const
+{
+	return (family_name_.defined() ? family_name_.value() : "");
+}
+
+
+
+string StorageDevice::get_serial_number() const
+{
+	return (serial_number_.defined() ? serial_number_.value() : "");
+}
+
+
+
+void StorageDevice::set_info_output(const string& s)
+{
+	info_output_ = s;
+}
+
+
+
+string StorageDevice::get_info_output() const
+{
+	return info_output_;
+}
+
+
+
+void StorageDevice::set_full_output(const string& s)
+{
+	full_output_ = s;
+}
+
+
+
+string StorageDevice::get_full_output() const
+{
+	return full_output_;
+}
+
+
+
+void StorageDevice::set_is_manually_added(bool b)
+{
+	is_manually_added_ = b;
+}
+
+
+
+bool StorageDevice::get_is_manually_added() const
+{
+	return is_manually_added_;
+}
+
+
+
+void StorageDevice::set_test_is_active(bool b)
+{
+	bool changed = (test_is_active_ != b);
+	test_is_active_ = b;
+	if (changed) {
+		signal_changed.emit(this);  // so that everybody stops any test-aborting operations.
+	}
+}
+
+
+
+bool StorageDevice::get_test_is_active() const
+{
+	return test_is_active_;
+}
+
+
+
+std::string StorageDevice::get_save_filename() const
+{
+	std::string model = this->get_model_name();  // may be empty
+	std::string serial = this->get_serial_number();
+	std::string date = hz::format_date("%Y-%m-%d", false);
+
+	std::string filename_format;
+	rconfig::get_data("gui/smartctl_output_filename_format", filename_format);
+	hz::string_replace(filename_format, "{serial}", serial);
+	hz::string_replace(filename_format, "{model}", model);
+	hz::string_replace(filename_format, "{date}", date);
+
+	return hz::filename_make_safe(filename_format);
+}
+
+
 
 std::string StorageDevice::get_device_options() const
 {
@@ -439,26 +769,31 @@ std::string StorageDevice::get_device_options() const
 		return std::string();
 	}
 
-	std::string config_options = app_get_device_option(get_device());
+	// If we have some special type or option, specify it on the command line (like "-d scsi").
+	// Note that the latter "-d" option overrides the former.
 
-	// If we have some special type, specify it on the command line (like "-d scsi").
-	// Note that the latter "-d" option overrides the former, so we're ok with multiple ones.
-	std::string type_arg = get_type_arg_name(this->get_type());
-
-	if (!type_arg.empty()) {
- 	if (!config_options.empty()) {
-			config_options += " ";
-		}
-		config_options += "-d " + type_arg;
+	// lowest priority - the detected type
+	std::vector<std::string> args;
+	if (!get_type_argument().empty()) {
+		args.push_back("-d " + get_type_argument());
+	}
+	// extra args, as specified manually in CLI or when adding the drive
+	if (!get_extra_arguments().empty()) {
+		args.push_back(get_extra_arguments());
 	}
 
-	return config_options;
+	// config options, as specified in preferences.
+	std::string config_options = app_get_device_option(get_device(), get_type_argument());
+	if (!config_options.empty()) {
+		args.push_back(config_options);
+	}
+
+	return hz::string_join(args, " ");
 }
 
 
 
-// Returns error message on error, empty string on success
-std::string StorageDevice::execute_smartctl(const std::string& command_options,
+std::string StorageDevice::execute_device_smartctl(const std::string& command_options,
 		hz::intrusive_ptr<CmdexSync> smartctl_ex, std::string& smartctl_output, bool check_type)
 {
 	// don't forbid running on currently tested drive - we need to call this from the test code.
@@ -470,69 +805,22 @@ std::string StorageDevice::execute_smartctl(const std::string& command_options,
 
 	std::string device = get_device();
 
-#ifndef _WIN32  // win32 doesn't have slashes in devices names
-	{
-		std::string::size_type pos = device.rfind('/');  // find basename
-		if (pos == std::string::npos) {
-			debug_out_error("app", DBG_FUNC_MSG << "Invalid device name \"" << device << "\".\n");
-			return "Invalid device name specified.";
-		}
-	}
-#endif
+	std::string error_msg = execute_smartctl(device, this->get_device_options(),
+			command_options, smartctl_ex, smartctl_output);
 
-	if (!smartctl_ex)  // if it doesn't exist, create a default one
-		smartctl_ex = new SmartctlExecutor();  // will be auto-deleted
-
-	std::string smartctl_binary = get_smartctl_binary();
-
-	if (smartctl_binary.empty()) {
-		debug_out_error("app", DBG_FUNC_MSG << "Smartctl binary is not set in config.\n");
-		return "Smartctl binary is not specified in configuration.";
-	}
-
-	std::string smartctl_def_options;
-	rconfig::get_data("system/smartctl_options", smartctl_def_options);
-
-	if (!smartctl_def_options.empty())
-		smartctl_def_options += " ";
-
-
-	std::string device_specific_options = this->get_device_options();
-	if (!device_specific_options.empty())
-		device_specific_options += " ";
-
-
-	smartctl_ex->set_command(Glib::shell_quote(smartctl_binary),
-			smartctl_def_options + device_specific_options + command_options + " " + Glib::shell_quote(device));
-
-	if (!smartctl_ex->execute() || !smartctl_ex->get_error_msg().empty()) {
+	if (!error_msg.empty()) {
 		debug_out_warn("app", DBG_FUNC_MSG << "Error while executing smartctl binary.\n");
 
-		std::string output = smartctl_ex->get_stdout_str();
-
-		// check if it's a device permission error.
-		// Smartctl open device: /dev/sdb failed: Permission denied
-		if (app_pcre_match("/Smartctl open device.+Permission denied/mi", output)) {
-			return "Permission denied while opening device.";
-		}
-
 		// Smartctl 5.39 cvs/svn version defaults to usb type on at least linux and windows.
 		// This means that the old SCSI identify command isn't executed by default,
 		// and there is no information about the device manufacturer/etc... in the output.
 		// We detect this and set the device type to scsi to at least have _some_ info.
-		if (check_type && this->get_type() == type_unknown
-				&& app_pcre_match("/specify device type with the -d option/mi", output)) {
-			this->set_type(type_invalid);
+		if (check_type && this->get_detected_type() == detected_type_unknown
+				&& app_pcre_match("/specify device type with the -d option/mi", smartctl_output)) {
+			this->set_detected_type(detected_type_invalid);
 		}
 
-		return smartctl_ex->get_error_msg();
-	}
-
-	// any_to_unix is needed for windows
-	smartctl_output = hz::string_trim_copy(hz::string_any_to_unix_copy(smartctl_ex->get_stdout_str()));
-	if (smartctl_output.empty()) {
-		debug_out_error("app", DBG_FUNC_MSG << "Smartctl returned an empty output.\n");
-		return "Smartctl returned an empty output.";
+		return error_msg;
 	}
 
 	return std::string();
@@ -540,6 +828,19 @@ std::string StorageDevice::execute_smartctl(const std::string& command_options,
 
 
 
+void StorageDevice::set_fully_parsed(bool b)
+{
+	fully_parsed_ = b;
+}
+
+
+
+void StorageDevice::set_properties(const SmartctlParser::prop_list_t& props)
+{
+	properties_ = props;
+}
+
+
 
 
 
