@@ -1,6 +1,6 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
@@ -27,16 +27,22 @@ inline device_option_map_t app_unserialize_device_option_map(const std::string&
 	hz::Bin2AsciiEncoder enc;
 
 	std::vector<std::string> pairs;
-	hz::string_split(str, ';', pairs, true);
+	hz::string_split(str, ";", pairs, true);
 
 	device_option_map_t option_map;
 	for (std::vector<std::string>::const_iterator iter = pairs.begin(); iter != pairs.end(); ++iter) {
 		std::vector<std::string> dev_entry;
-		hz::string_split(*iter, ':', dev_entry, true);
+		hz::string_split(*iter, ":", dev_entry, true, 2);
 
+		std::string dev, opt;
 		if (dev_entry.size() == 2) {
-			std::string dev = hz::string_trim_copy(enc.decode(dev_entry[0]));
-			std::string opt = hz::string_trim_copy(enc.decode(dev_entry[1]));
+			dev = dev_entry.at(0);  // includes type (separated by encoded "::")
+			opt = dev_entry.at(1);
+		}
+
+		if (dev != "") {
+			dev = hz::string_trim_copy(enc.decode(dev));
+			opt = hz::string_trim_copy(enc.decode(opt));
 
 			// ignore potentially harmful chars
 			if (!dev.empty() && !opt.empty()
@@ -62,13 +68,13 @@ inline std::string app_serialize_device_option_map(const device_option_map_t& op
 			pairs.push_back(enc.encode(iter->first) + ":" + enc.encode(iter->second));
 	}
 
-	return hz::string_join(pairs, ';');
+	return hz::string_join(pairs, ";");
 }
 
 
 
 
-inline std::string app_get_device_option(const std::string& dev)
+inline std::string app_get_device_option(const std::string& dev, const std::string& type_arg)
 {
 	if (dev.empty())
 		return std::string();
@@ -79,11 +85,27 @@ inline std::string app_get_device_option(const std::string& dev)
 
 	device_option_map_t devmap = app_unserialize_device_option_map(devmap_str);
 
-	device_option_map_t::const_iterator iter = devmap.find(dev);
-	if (iter == devmap.end())
-		return std::string();
+	// try the concrete type first
+	if (!type_arg.empty()) {
+		device_option_map_t::const_iterator iter = devmap.find(dev + "::" + type_arg);
+		if (iter != devmap.end()) {
+			return iter->second;
+		}
+	}
 
-	return iter->second;
+	// in case there's a trailing delimiter
+	device_option_map_t::const_iterator iter = devmap.find(dev + "::" + type_arg);
+	if (iter != devmap.end()) {
+		return iter->second;
+	}
+
+	// just the device name
+	iter = devmap.find(dev);
+	if (iter != devmap.end()) {
+		return iter->second;
+	}
+
+	return std::string();
 }
 
 
