@@ -1,6 +1,6 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
@@ -11,7 +11,7 @@
 
 #include "hz/down_cast.h"
 #include "hz/string_num.h"  // number_to_string
-#include "hz/string_format.h"  // string_sprintf
+#include "hz/string_sprintf.h"  // string_sprintf
 #include "hz/string_algo.h"  // string_join
 #include "hz/fs_file.h"  // hz::File
 #include "hz/format_unit.h"  // format_time_length
@@ -76,7 +76,7 @@ namespace {
 				label->set_flags(label->get_flags() & ~Gtk::CAN_FOCUS);
 
 				std::string fg;
-				if (app_property_get_label_hilight_color(iter->property->warning, fg))
+				if (app_property_get_label_highlight_color(iter->property->warning, fg))
 					label->set_markup("<span color=\"" + fg + "\">" + label_text + "</span>");
 				vbox->pack_start(*label, false, false);
 
@@ -100,7 +100,7 @@ namespace {
 		Gtk::CellRendererText* crt = hz::down_cast<Gtk::CellRendererText*>(cr);
 		if (crt) {
 			std::string fg, bg;
-			if (app_property_get_row_hilight_colors(p->warning, fg, bg)) {
+			if (app_property_get_row_highlight_colors(p->warning, fg, bg)) {
 				// Note: property_cell_background makes horizontal tree lines disappear around it,
 				// but property_background doesn't play nice with sorted column color.
 				crt->property_cell_background() = bg;
@@ -115,7 +115,7 @@ namespace {
 	}
 
 
-	inline void app_hilight_tab_label(Gtk::Widget* label_widget,
+	inline void app_highlight_tab_label(Gtk::Widget* label_widget,
 			StorageProperty::warning_t warning, const Glib::ustring& original_label)
 	{
 		Gtk::Label* label = hz::down_cast<Gtk::Label*>(label_widget);
@@ -128,7 +128,7 @@ namespace {
 		}
 
 		std::string fg;
-		if (app_property_get_label_hilight_color(warning, fg))
+		if (app_property_get_label_highlight_color(warning, fg))
 			label->set_markup_with_mnemonic("<span color=\"" + fg + "\">" + original_label + "</span>");
 	}
 
@@ -161,7 +161,7 @@ namespace {
 // glade/gtkbuilder needs this constructor
 GscInfoWindow::GscInfoWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui)
 		: AppUIResWidget<GscInfoWindow, true>(gtkcobj, ref_ui),
-		device_name_label(0)
+		device_name_label(0), test_force_bar_update(true)
 {
 
 	// Create missing widgets
@@ -273,7 +273,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		// fetch all smartctl info, even if it already has it (to refresh it).
 		if (scan) {
 			SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-			ex->create_running_dialog(this, "Running %s on " + drive->get_device() + "...");
+			ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 			std::string error_msg = drive->fetch_data_and_parse(ex);  // run it with GUI support
 
 			if (!error_msg.empty()) {
@@ -281,12 +281,13 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 				return;
 			}
 
-		} else {
-			std::string error_msg = drive->parse_data();
-			if (!error_msg.empty()) {
-				gui_show_error_dialog("Cannot interpret SMART data", error_msg, this);
-				return;
-			}
+		// it's already parsed
+// 		} else {
+// 			std::string error_msg = drive->parse_data();
+// 			if (!error_msg.empty()) {
+// 				gui_show_error_dialog("Cannot interpret SMART data", error_msg, this);
+// 				return;
+// 			}
 		}
 	}
 
@@ -307,28 +308,28 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 
 	if (hide_tabs) {
 		bool smart_enabled = (drive->get_smart_status() == StorageDevice::status_enabled);
-		Gtk::Widget* note_label = 0;
+		Gtk::Widget* note_page_box = 0;
 
-		if ((note_label = lookup_widget("capabilities_tab_vbox")) != 0) {
-			if (smart_enabled) { note_label->show(); } else { note_label->hide(); }
+		if ((note_page_box = lookup_widget("capabilities_tab_vbox")) != 0) {
+			if (smart_enabled) { note_page_box->show(); } else { note_page_box->hide(); }
 		}
-		if ((note_label = lookup_widget("attributes_tab_vbox")) != 0) {
-			if (smart_enabled) { note_label->show(); } else { note_label->hide(); }
+		if ((note_page_box = lookup_widget("attributes_tab_vbox")) != 0) {
+			if (smart_enabled) { note_page_box->show(); } else { note_page_box->hide(); }
 		}
-		if ((note_label = lookup_widget("error_log_tab_vbox")) != 0) {
-			if (smart_enabled) { note_label->show(); } else { note_label->hide(); }
+		if ((note_page_box = lookup_widget("error_log_tab_vbox")) != 0) {
+			if (smart_enabled) { note_page_box->show(); } else { note_page_box->hide(); }
 		}
-		if ((note_label = lookup_widget("selftest_log_tab_vbox")) != 0) {
-			if (smart_enabled) { note_label->show(); } else { note_label->hide(); }
+		if ((note_page_box = lookup_widget("selftest_log_tab_vbox")) != 0) {
+			if (smart_enabled) { note_page_box->show(); } else { note_page_box->hide(); }
 		}
-		if ((note_label = lookup_widget("test_tab_vbox")) != 0) {
-			if (smart_enabled) { note_label->show(); } else { note_label->hide(); }
+		if ((note_page_box = lookup_widget("test_tab_vbox")) != 0) {
+			if (smart_enabled) { note_page_box->show(); } else { note_page_box->hide(); }
 		}
 	}
 
 
 	{
-		std::string device = drive->get_device_pretty(true);
+		std::string device = drive->get_device_pretty();
 		std::string model = (drive->get_model_name().empty() ? "Unknown model" : drive->get_model_name());
 
 		this->set_title("Device Information - " + device + ": " + model + " - GSmartControl");
@@ -357,8 +358,10 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 
 		for (prop_iterator iter = props.begin(); iter != props.end(); ++iter) {
 			if (iter->section == StorageProperty::section_info) {
-				if (iter->generic_name == "smartctl_version") {
+				if (iter->generic_name == "smartctl_version_full") {
 					version_props.push_back(*iter);
+				} else if (iter->generic_name == "smartctl_version") {
+					continue;  // we use the full version string instead.
 				} else {
 					id_props.push_back(*iter);
 				}
@@ -410,7 +413,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 			value->set_markup(Glib::Markup::escape_text(iter->format_value()));
 
 			std::string fg;
-			if (app_property_get_label_hilight_color(iter->warning, fg)) {
+			if (app_property_get_label_highlight_color(iter->warning, fg)) {
 				name->set_markup("<span color=\"" + fg + "\">"+ name->get_label() + "</span>");
 				value->set_markup("<span color=\"" + fg + "\">"+ value->get_label() + "</span>");
 			}
@@ -433,7 +436,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		identity_table->show_all();
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("identity_tab_label"), max_warning, tab_identity_name);
+		app_highlight_tab_label(lookup_widget("identity_tab_label"), max_warning, tab_identity_name);
 
 	} while (false);
 
@@ -528,7 +531,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("capabilities_tab_label"), max_warning, tab_capabilities_name);
+		app_highlight_tab_label(lookup_widget("capabilities_tab_label"), max_warning, tab_capabilities_name);
 
 	} while (false);
 
@@ -602,7 +605,15 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		Gtk::TreeModelColumn<std::string> col_flag_value;
 		model_columns.add(col_flag_value);
 		num_tree_cols = app_gtkmm_create_tree_view_column(col_flag_value, *treeview,
-				"Flag", "Flag value", false);
+				"Flag", "Flag value\n\n"
+				"If given in POSRCK+ format, the presence of each letter indicates that the flag is on.\n"
+				"P: pre-failure warning\n"
+				"O: updated continuously\n"
+				"S: speed / performance attribute\n"
+				"R: error rate\n"
+				"C: event count\n"
+				"K: auto-keep\n"
+				"+: undocumented bits present", false);
 
 		Gtk::TreeModelColumn<Glib::ustring> col_tooltip;
 		model_columns.add(col_tooltip);
@@ -656,9 +667,9 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 			row[col_id] = iter->value_attribute.id;
 			row[col_name] = iter->readable_name;
 			row[col_flag_value] = iter->value_attribute.flag;  // it's a string, not int.
-			row[col_value] = hz::number_to_string(iter->value_attribute.value);
-			row[col_worst] = hz::number_to_string(iter->value_attribute.worst);
-			row[col_threshold] = hz::number_to_string(iter->value_attribute.threshold);
+			row[col_value] = (iter->value_attribute.value.defined() ? hz::number_to_string(iter->value_attribute.value.value()) : "-");
+			row[col_worst] = (iter->value_attribute.worst.defined() ? hz::number_to_string(iter->value_attribute.worst.value()) : "-");
+			row[col_threshold] = (iter->value_attribute.threshold.defined() ? hz::number_to_string(iter->value_attribute.threshold.value()) : "-");
 			row[col_raw] = iter->value_attribute.raw_value;
 			row[col_type] = attr_type;
 			row[col_updated] = StorageAttribute::get_update_type_name(iter->value_attribute.update_type);
@@ -677,7 +688,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		app_set_top_labels(label_vbox, label_strings);
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("attributes_tab_label"), max_warning, tab_attributes_name);
+		app_highlight_tab_label(lookup_widget("attributes_tab_label"), max_warning, tab_attributes_name);
 
 	} while (false);
 
@@ -799,9 +810,11 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 				row[col_num] = iter->value_error_block.error_num;
 				row[col_hours] = iter->value_error_block.lifetime_hours;
 				row[col_state] = iter->value_error_block.device_state;
-				row[col_type] = StorageErrorBlock::get_readable_error_type(iter->value_error_block.reported_type);
+				row[col_type] = StorageErrorBlock::get_readable_error_types(iter->value_error_block.reported_types);
 				row[col_details] = (type_details.empty() ? "-" : type_details);  // e.g. OBS has no details
-				row[col_tooltip] = iter->get_description();
+				// There are no descriptions in self-test log entries, so don't display
+				// "No description available" for all of them.
+// 				row[col_tooltip] = iter->get_description();
 				row[col_storage] = &(*iter);
 				row[col_mark_name] = "Error " + hz::number_to_string(iter->value_error_block.error_num);
 			}
@@ -815,7 +828,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		app_set_top_labels(label_vbox, label_strings);
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("error_log_tab_label"), max_warning, tab_error_log_name);
+		app_highlight_tab_label(lookup_widget("error_log_tab_label"), max_warning, tab_error_log_name);
 
 	} while (false);
 
@@ -916,7 +929,9 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 			row[col_percent] = hz::number_to_string(100 - iter->value_selftest_entry.remaining_percent) + "%";
 			row[col_hours] = iter->value_selftest_entry.lifetime_hours;
 			row[col_lba] = iter->value_selftest_entry.lba_of_first_error;
-			row[col_tooltip] = iter->get_description();
+			// There are no descriptions in self-test log entries, so don't display
+			// "No description available" for all of them.
+			// row[col_tooltip] = iter->get_description();
 			row[col_storage] = &(*iter);
 
 			if (int(iter->warning) > int(max_warning))
@@ -928,7 +943,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 		app_set_top_labels(label_vbox, label_strings);
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("selftest_log_tab_label"), max_warning, tab_selftest_log_name);
+		app_highlight_tab_label(lookup_widget("selftest_log_tab_label"), max_warning, tab_selftest_log_name);
 
 	} while (false);
 
@@ -1024,7 +1039,7 @@ void GscInfoWindow::fill_ui_with_info(bool scan, bool clear_ui, bool clear_tests
 			row = *(test_combo_model->append());
 			row[test_combo_col_name] = SelfTest::get_test_name(SelfTest::type_long);
 			row[test_combo_col_description] =
-				"Extended self-test examines complete disk surface and performs various test routines "
+				"Extended self-test examines complete disk surface and performs various test routines"
 				" built into the drive. Its result is reported in the Self-test Log.";
 			row[test_combo_col_self_test] = test_long;
 		}
@@ -1086,7 +1101,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("identity_tab_label"), StorageProperty::warning_none, tab_identity_name);
+		app_highlight_tab_label(lookup_widget("identity_tab_label"), StorageProperty::warning_none, tab_identity_name);
 	}
 
 	{
@@ -1102,7 +1117,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("capabilities_tab_label"), StorageProperty::warning_none, tab_capabilities_name);
+		app_highlight_tab_label(lookup_widget("capabilities_tab_label"), StorageProperty::warning_none, tab_capabilities_name);
 	}
 
 	{
@@ -1119,7 +1134,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("attributes_tab_label"), StorageProperty::warning_none, tab_attributes_name);
+		app_highlight_tab_label(lookup_widget("attributes_tab_label"), StorageProperty::warning_none, tab_attributes_name);
 	}
 
 	{
@@ -1143,7 +1158,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("error_log_tab_label"), StorageProperty::warning_none, tab_error_log_name);
+		app_highlight_tab_label(lookup_widget("error_log_tab_label"), StorageProperty::warning_none, tab_error_log_name);
 	}
 
 	{
@@ -1160,7 +1175,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 		}
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("selftest_log_tab_label"), StorageProperty::warning_none, tab_selftest_log_name);
+		app_highlight_tab_label(lookup_widget("selftest_log_tab_label"), StorageProperty::warning_none, tab_selftest_log_name);
 	}
 
 	{
@@ -1216,7 +1231,7 @@ void GscInfoWindow::clear_ui_info(bool clear_tests_too)
 
 
 		// tab label
-		app_hilight_tab_label(lookup_widget("test_tab_label"), StorageProperty::warning_none, tab_test_name);
+		app_highlight_tab_label(lookup_widget("test_tab_label"), StorageProperty::warning_none, tab_test_name);
 	}
 
 }
@@ -1251,7 +1266,17 @@ void GscInfoWindow::on_view_output_button_clicked()
 {
 	GscTextWindow<SmartctlOutputInstance>* win = GscTextWindow<SmartctlOutputInstance>::create();
 	// make save visible and enable monospace font
-	win->set_text("Smartctl Output", this->drive->get_full_output(), true, true);
+
+	std::string buf_text = this->drive->get_full_output();
+	// We receive locale'd thousands separators in win32, so convert them.
+	#ifdef _WIN32
+	try {
+		buf_text = Glib::locale_to_utf8(buf_text);
+	} catch (Glib::ConvertError& e) {
+		buf_text = "";  // inserting invalid utf8 may trigger a segfault, so empty better.
+	}
+	#endif
+	win->set_text("Smartctl Output", buf_text, true, true);
 
 	std::string filename = drive->get_save_filename();
 	if (!filename.empty())
@@ -1354,115 +1379,117 @@ void GscInfoWindow::on_test_type_combo_changed()
 
 
 
+
 // Note: Another loop like this may run inside it for another drive.
-void GscInfoWindow::test_loop()
+gboolean GscInfoWindow::test_idle_callback(void* data)
 {
-	if (!current_test)  // shouldn't happen
-		return;
+	GscInfoWindow* self = static_cast<GscInfoWindow*>(data);
+	DBG_ASSERT(self);
 
-	// Switch GUI to running test mode
+	if (!self->current_test)  // shouldn't happen
+		return false;  // stop
 
-	Gtk::ComboBox* test_type_combo = lookup_widget<Gtk::ComboBox*>("test_type_combo");
-	if (test_type_combo)
-		test_type_combo->set_sensitive(false);
-
-	Gtk::Button* test_execute_button = lookup_widget<Gtk::Button*>("test_execute_button");
-	if (test_execute_button)
-		test_execute_button->set_sensitive(false);
+	Gtk::ProgressBar* test_completion_progressbar =
+			self->lookup_widget<Gtk::ProgressBar*>("test_completion_progressbar");
 
 
-	Gtk::ProgressBar* test_completion_progressbar = lookup_widget<Gtk::ProgressBar*>("test_completion_progressbar");
-	if (test_completion_progressbar) {
-		test_completion_progressbar->set_text("");
-		test_completion_progressbar->set_sensitive(true);
-		test_completion_progressbar->show();
-	}
+	bool active = true;
 
-	Gtk::Button* test_stop_button = lookup_widget<Gtk::Button*>("test_stop_button");
-	if (test_stop_button) {
-		test_stop_button->set_sensitive(true);  // true while test is active
-		test_stop_button->show();
-	}
-
-	std::string error_msg;  // our errors
-
-	Glib::Timer timer_poll, timer_bar;
-
-	// one update() is performed by start(), so do the timeout first
-	while (true) {
-		if (!current_test->is_active())  // check status
+	do {  // goto
+		if (!self->current_test->is_active()) {  // check status
+			active = false;
 			break;
+		}
 
-		int8_t rem_percent = current_test->get_remaining_percent();
+		int8_t rem_percent = self->current_test->get_remaining_percent();
 		std::string rem_percent_str = (rem_percent == -1 ? "Unknown" : hz::number_to_string(100 - rem_percent));
 
-		timer_poll.start();  // restart it
-		int64_t poll_in = current_test->get_poll_in_seconds();  // sec
+		int64_t poll_in = self->current_test->get_poll_in_seconds();  // sec
 
-		timer_bar.start();  // restart
-		bool force_bar_update = true;  // every time after poll
 
-		// wait until next poll (up to several minutes)
-		while (timer_poll.elapsed() < static_cast<double>(poll_in)) {  // elapsed() is seconds in double.
+		// One update() is performed by start(), so do the timeout first.
 
-			Glib::usleep(30*1000);  // 30msec. don't do more - we may have two or more loops like this
+		// Wait until next poll (up to several minutes). Meanwhile, interpolate
+		// the remaining time, update the progressbar, etc...
+		if (self->test_timer_poll.elapsed() < static_cast<double>(poll_in)) {  // elapsed() is seconds in double.
 
-			if (force_bar_update || timer_bar.elapsed() >= 5.) {  // update progress bar every 5 seconds or after poll.
+			// Update progress bar right after poll, plus every 5 seconds.
+			if (self->test_force_bar_update || self->test_timer_bar.elapsed() >= 5.) {
 
-				int64_t rem_seconds = current_test->get_remaining_seconds();
-				std::string rem_seconds_str = (rem_seconds == -1 ? "Unknown" : hz::format_time_length(rem_seconds));
-
-				Glib::ustring bar_str = hz::string_sprintf("Test completion: %s%%; ETA: %s",
-						rem_percent_str.c_str(), rem_seconds_str.c_str());
-				if (!error_msg.empty())
-					bar_str = error_msg;  // better than popup every few seconds
+				int64_t rem_seconds = self->current_test->get_remaining_seconds();
 
 				if (test_completion_progressbar) {
+					std::string rem_seconds_str = (rem_seconds == -1 ? "Unknown" : hz::format_time_length(rem_seconds));
+
+					Glib::ustring bar_str;
+
+					if (self->test_error_msg.empty()) {
+						bar_str = hz::string_sprintf("Test completion: %s%%; ETA: %s",
+								rem_percent_str.c_str(), rem_seconds_str.c_str());
+					} else {
+						bar_str = self->test_error_msg;  // better than popup every few seconds
+					}
+
 					test_completion_progressbar->set_text(bar_str);
 					test_completion_progressbar->set_fraction(std::max(0., std::min(1., 1. - (rem_percent / 100.))));
 				}
 
-				force_bar_update = false;
-				timer_bar.start();  // restart
+				self->test_force_bar_update = false;
+				self->test_timer_bar.start();  // restart
 			}
 
-			while (Gtk::Main::events_pending()) {
-				Gtk::Main::iteration();
-			}
-
-			if (!current_test->is_active())  // check status
+			if (!self->current_test->is_active()) {  // check status
+				active = false;
 				break;
+			}
+
+
+		} else {  // it's poll time
+
+			if (!self->current_test->is_active()) {  // the inner loop stopped, stop this one too
+				active = false;
+				break;
+			}
+
+			SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
+			ex->create_running_dialog(self);
+
+			self->test_error_msg = self->current_test->update(ex);
+			if (!self->test_error_msg.empty()) {
+// 				gui_show_error_dialog("Cannot monitor test progress", self->test_error_msg, this);  // better show in progressbar.
+				self->current_test->force_stop(ex);  // what else can we do?
+				active = false;
+				break;
+			}
+
+			self->test_timer_poll.start();  // restart it
+			self->test_force_bar_update = true;  // force progressbar / ETA update on the next tick
 		}
 
-		if (!current_test->is_active())  // the inner loop stopped, stop this one too
-			break;
 
-		SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-		ex->create_running_dialog(this);
+	} while (false);
 
-		error_msg = current_test->update(ex);
-		if (!error_msg.empty()) {
-// 			gui_show_error_dialog("Cannot monitor test progress", error_msg, this);  // better show in progressbar.
-			current_test->force_stop(ex);  // what else can we do?
-		}
+
+	if (active) {
+		return true;  // continue the idle callback
 	}
 
-	timer_poll.stop();  // just in case
-	timer_bar.stop();  // just in case
 
+	// Test is finished, clean up
 
-	// Test is finished
+	self->test_timer_poll.stop();  // just in case
+	self->test_timer_bar.stop();  // just in case
 
-	StorageSelftestEntry::status_t status = current_test->get_status();
+	StorageSelftestEntry::status_t status = self->current_test->get_status();
 
 	bool aborted = false;
 	StorageSelftestEntry::status_severity_t severity = StorageSelftestEntry::severity_none;
 	std::string result_msg;
 
-	if (!error_msg.empty()) {
+	if (!self->test_error_msg.empty()) {
 		aborted = true;
 		severity = StorageSelftestEntry::severity_error;
-		result_msg = "<b>Test aborted: </b>" + error_msg;
+		result_msg = "<b>Test aborted: </b>" + self->test_error_msg;
 
 	} else {
 		severity = StorageSelftestEntry::get_status_severity(status);
@@ -1484,15 +1511,18 @@ void GscInfoWindow::test_loop()
 	}
 
 
+	Gtk::ComboBox* test_type_combo = self->lookup_widget<Gtk::ComboBox*>("test_type_combo");
 	if (test_type_combo)
 		test_type_combo->set_sensitive(true);
 
+	Gtk::Button* test_execute_button = self->lookup_widget<Gtk::Button*>("test_execute_button");
 	if (test_execute_button)
 		test_execute_button->set_sensitive(true);
 
 	if (test_completion_progressbar)
 		test_completion_progressbar->set_text(aborted ? "Test aborted" : "Test completed");
 
+	Gtk::Button* test_stop_button = self->lookup_widget<Gtk::Button*>("test_stop_button");
 	if (test_stop_button)
 		test_stop_button->set_sensitive(false);
 
@@ -1503,28 +1533,31 @@ void GscInfoWindow::test_loop()
 		stock_id = Gtk::Stock::DIALOG_WARNING;
 	}
 
-	Gtk::Image* test_result_image = lookup_widget<Gtk::Image*>("test_result_image");
+	Gtk::Image* test_result_image = self->lookup_widget<Gtk::Image*>("test_result_image");
 	// we use large icon size here because the icons we use are from dialogs.
 	// unfortunately, there are no non-dialog icons of this sort.
 	if (test_result_image)
 		test_result_image->set(stock_id, Gtk::ICON_SIZE_DND);
 
 
-	Gtk::Label* test_result_label = lookup_widget<Gtk::Label*>("test_result_label");
+	Gtk::Label* test_result_label = self->lookup_widget<Gtk::Label*>("test_result_label");
 	if (test_result_label)
 		test_result_label->set_markup(result_msg);
 
-	Gtk::HBox* test_result_hbox = lookup_widget<Gtk::HBox*>("test_result_hbox");
+	Gtk::HBox* test_result_hbox = self->lookup_widget<Gtk::HBox*>("test_result_hbox");
 	if (test_result_hbox)
 		test_result_hbox->show();
 
 
-	this->refresh_info(false);  // don't clear the tests tab
+	self->refresh_info(false);  // don't clear the tests tab
 
+	return false;  // stop idle callback
 }
 
 
 
+
+
 void GscInfoWindow::on_test_execute_button_clicked()
 {
 	Gtk::ComboBox* test_type_combo = lookup_widget<Gtk::ComboBox*>("test_type_combo");
@@ -1539,6 +1572,11 @@ void GscInfoWindow::on_test_execute_button_clicked()
 	if (!test)
 		return;
 
+	// hide previous test results from GUI
+	Gtk::HBox* test_result_hbox = this->lookup_widget<Gtk::HBox*>("test_result_hbox");
+	if (test_result_hbox)
+		test_result_hbox->hide();
+
 	SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
 	ex->create_running_dialog(this);
 
@@ -1550,8 +1588,51 @@ void GscInfoWindow::on_test_execute_button_clicked()
 
 	current_test = test;
 
-	// The Main Test Loop (tm)
-	test_loop();
+
+	// Switch GUI to "running test" mode
+
+	if (test_type_combo)
+		test_type_combo->set_sensitive(false);
+
+	Gtk::Button* test_execute_button = lookup_widget<Gtk::Button*>("test_execute_button");
+	if (test_execute_button)
+		test_execute_button->set_sensitive(false);
+
+
+	Gtk::ProgressBar* test_completion_progressbar = lookup_widget<Gtk::ProgressBar*>("test_completion_progressbar");
+	if (test_completion_progressbar) {
+		test_completion_progressbar->set_text("");
+		test_completion_progressbar->set_sensitive(true);
+		test_completion_progressbar->show();
+	}
+
+	Gtk::Button* test_stop_button = lookup_widget<Gtk::Button*>("test_stop_button");
+	if (test_stop_button) {
+		test_stop_button->set_sensitive(true);  // true while test is active
+		test_stop_button->show();
+	}
+
+
+	// reset these
+	test_error_msg = "";
+	test_timer_poll.start();
+	test_timer_bar.start();
+	test_force_bar_update = true;
+
+
+	// We don't use idle function here, because it has the following problem:
+	// CmdexSync::execute() (which is called on force_stop()) calls g_main_context_pending(),
+	// which returns true EVERY time, until the idle callback returns false.
+	// So, force_stop() exits its "execute abort" command only when the
+	// idle callback polls the drive on the next timeout and sees that the test
+	// has been actually aborted.
+	// So, we use timeout callback - and hope that there are no usleeps
+	// with 300ms so that g_main_context_pending() returns false at least once,
+	// to escape the execute() loop.
+
+// 	g_idle_add(test_idle_callback, this);
+	g_timeout_add(300, test_idle_callback, this);  // every 300ms
+
 }
 
 
@@ -1571,7 +1652,7 @@ void GscInfoWindow::on_test_stop_button_clicked()
 		return;
 	}
 
-	// nothing else to do - the cleanup is performed by test_loop.
+	// nothing else to do - the cleanup is performed by the idle callback.
 }
 
 
@@ -1600,7 +1681,7 @@ void GscInfoWindow::on_drive_changed(StorageDevice* pdrive)
 
 	// test_active is also checked in delete_event handler, because this call may not
 	// succeed - the window manager may refuse to do it.
-#if !APP_GTKMM_CHECK_VERSION(2, 10, 0)
+#if APP_GTKMM_CHECK_VERSION(2, 10, 0)
 	this->set_deletable(!test_active);  // since 2.10
 #endif
 }
