@@ -1,6 +1,6 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
@@ -15,9 +15,10 @@
 #include "applib/storage_detector.h"
 #include "applib/smartctl_parser.h"
 #include "applib/gui_utils.h"  // gui_show_error_dialog
+#include "applib/smartctl_executor.h"  // get_smartctl_binary()
 #include "applib/smartctl_executor_gui.h"
 #include "applib/app_gtkmm_utils.h"  // app_gtkmm_*
-#include "applib/storage_property_colors.h"  // app_property_get_label_hilight_color
+#include "applib/storage_property_colors.h"  // app_property_get_label_highlight_color
 #include "applib/app_pcrecpp.h"  // app_pcre_match
 
 #include "gsc_init.h"  // app_quit()
@@ -30,6 +31,8 @@
 
 #include "gsc_main_window_iconview.h"
 #include "gsc_main_window.h"
+#include "gsc_add_device_window.h"
+#include "applib/executor_factory.h"
 
 
 
@@ -40,7 +43,8 @@
 
 GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui)
 		: AppUIResWidget<GscMainWindow, false>(gtkcobj, ref_ui), iconview(0),
-		action_handling_enabled_(true), name_label(0), health_label(0), family_label(0)
+		action_handling_enabled_(true), name_label(0), health_label(0), family_label(0),
+		scanning_(false)
 {
 	APP_GTKMM_CONNECT_VIRTUAL(delete_event);  // make sure the event handler is called
 
@@ -60,9 +64,12 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 	bool show_output_button = true;
 
 	do {
-		std::string smartctl_binary, smartctl_def_options;
-		rconfig::get_data("system/smartctl_binary", smartctl_binary);
-		rconfig::get_data("system/smartctl_options", smartctl_def_options);
+		std::string smartctl_binary = get_smartctl_binary();
+
+		// Don't use default options here - they are used when invoked
+		// with a device option.
+// 		std::string smartctl_def_options;
+// 		rconfig::get_data("system/smartctl_options", smartctl_def_options);
 
 		if (smartctl_binary.empty()) {
 			error_msg = "Smartctl binary is not specified in configuration.";
@@ -70,14 +77,15 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 			break;
 		}
 
-		if (!smartctl_def_options.empty())
-			smartctl_def_options += " ";
+// 		if (!smartctl_def_options.empty())
+// 			smartctl_def_options += " ";
 
 		SmartctlExecutorGui ex;
 		ex.create_running_dialog(this);
 		ex.set_running_msg("Checking if smartctl is executable...");
 
-		ex.set_command(Glib::shell_quote(smartctl_binary), smartctl_def_options + " -V");  // --version
+// 		ex.set_command(Glib::shell_quote(smartctl_binary), smartctl_def_options + "-V");  // --version
+		ex.set_command(Glib::shell_quote(smartctl_binary), "-V");  // --version
 
 		if (!ex.execute() || !ex.get_error_msg().empty()) {
 			error_msg = ex.get_error_msg();
@@ -90,33 +98,38 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 			break;
 		}
 
-		if (SmartctlParser::parse_version(output).empty()) {
+		std::string version, version_full;
+		if (!SmartctlParser::parse_version(output, version, version_full)) {
 			error_msg = "Smartctl returned invalid output.";
 			break;
 		}
 
 	} while (false);
 
-	if (!error_msg.empty()) {
+	bool smartctl_valid = error_msg.empty();
+	if (!smartctl_valid) {
 		gsc_executor_error_dialog_show("There was an error while executing smartctl",
 				error_msg + "\n\n<i>Please specify the correct smartctl binary in Preferences.</i>",
-				this, show_output_button, true);
-		return;
+				this, true, show_output_button);
 	}
 
-
 	// Scan
-
-	populate_iconview();
+	populate_iconview(smartctl_valid);
 
 }
 
 
 
 
-void GscMainWindow::populate_iconview()
+void GscMainWindow::populate_iconview(bool smartctl_valid)
 {
-	if (rconfig::get_data<bool>("gui/scan_on_startup")  // config option
+	if (!smartctl_valid) {
+		iconview->empty_view_message = GscMainWindowIconView::message_no_smartctl;
+		iconview->clear_all();  // the message won't be shown without invalidating the region.
+		while (Gtk::Main::events_pending())  // give expose event the time it needs
+			Gtk::Main::iteration();
+
+	} else if (rconfig::get_data<bool>("gui/scan_on_startup")  // config option
 			&& !rconfig::get_data<bool>("/runtime/gui/force_no_scan_on_startup")) {  // command-line option
 		rescan_devices();  // scan for devices and fill the iconview
 
@@ -129,11 +142,20 @@ void GscMainWindow::populate_iconview()
 
 	// Add command-line-requested devices and virtual drives.
 
-	std::vector<std::string> load_devices;
-	if (rconfig::get_data("/runtime/gui/add_devices_on_startup", load_devices)) {
-		for (unsigned int i = 0; i < load_devices.size(); ++i) {
-			if (!load_devices[i].empty()) {
-				add_device(load_devices[i]);
+	if (smartctl_valid) {
+		std::vector<std::string> load_devices;
+		if (rconfig::get_data("/runtime/gui/add_devices_on_startup", load_devices)) {
+			for (unsigned int i = 0; i < load_devices.size(); ++i) {
+				if (!load_devices[i].empty()) {
+					std::vector<std::string> parts;
+					hz::string_split(load_devices[i], "::", parts, false);
+					std::string file = (parts.size() > 0 ? parts.at(0) : "");
+					std::string type_arg = (parts.size() > 1 ? parts.at(1) : "");
+					std::string extra_args = (parts.size() > 2 ? parts.at(2) : "");
+					if (!file.empty()) {
+						add_device(file, type_arg, extra_args);
+					}
+				}
 			}
 		}
 	}
@@ -237,8 +259,8 @@ bool GscMainWindow::create_widgets()
 	actiongroup_main->add(Gtk::Action::create("file_menu", "_File"));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_quit), Gtk::Stock::QUIT);
-		actiongroup_main->add(action, Gtk::AccelKey("<control>Q"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_quit), action));
+		actiongroup_main->add((action_map[action_quit] = action), Gtk::AccelKey("<control>Q"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_quit));
 
 	actiongroup_main->add(Gtk::Action::create("device_menu", "_Device"));
 
@@ -248,88 +270,88 @@ bool GscMainWindow::create_widgets()
 #else
 		action = Gtk::Action::create(APP_ACTION_NAME(action_view_details), "_View details", "View detailed information");
 #endif
-		actiongroup_device->add(action, Gtk::AccelKey("<control>V"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_view_details), action));
+		actiongroup_device->add((action_map[action_view_details] = action), Gtk::AccelKey("<control>V"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_view_details));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_smart), "Enable S_MART",
 				"Toggle SMART status. The status will be preserved at least until reboot (unless you toggle it again).");
 		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_smart_enabled_check"));
-		actiongroup_device->add(action, Gtk::AccelKey("<control>M"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_smart), action));
+		actiongroup_device->add((action_map[action_enable_smart] = action), Gtk::AccelKey("<control>M"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_smart));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_aodc), "Enable Auto O_ffline Data Collection",
 				"Toggle Automatic Offline Data Collection which will update \"offline\" SMART attributes every four hours");
 		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_aodc_enabled_check"));
-		actiongroup_device->add(action, Gtk::AccelKey("<control>F"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_aodc), action));
+		actiongroup_device->add((action_map[action_enable_aodc] = action), Gtk::AccelKey("<control>F"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_aodc));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_reread_device_data), Gtk::Stock::REFRESH, "R_e-read Data",
 				"Re-read basic SMART data");
-		actiongroup_device->add(action, Gtk::AccelKey("<control>E"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_reread_device_data), action));
+		actiongroup_device->add((action_map[action_reread_device_data] = action), Gtk::AccelKey("<control>E"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_reread_device_data));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_perform_tests), "Perform _Tests...",
 				"Perform various self-tests on the drive");
-		actiongroup_device->add(action, Gtk::AccelKey("<control>T"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_perform_tests), action));
+		actiongroup_device->add((action_map[action_perform_tests] = action), Gtk::AccelKey("<control>T"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_perform_tests));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_remove_device), Gtk::Stock::REMOVE, "Re_move Added Device",
 				"Remove previously added device");
-		actiongroup_device->add(action, Gtk::AccelKey("<control>W"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_remove_device), action));
+		actiongroup_device->add((action_map[action_remove_device] = action), Gtk::AccelKey("<control>W"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_remove_device));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_remove_virtual_device), Gtk::Stock::REMOVE, "Re_move Virtual Device",
 				"Remove previously loaded virtual device");
-		actiongroup_device->add(action, Gtk::AccelKey("Delete"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_remove_virtual_device), action));
+		actiongroup_device->add((action_map[action_remove_virtual_device] = action), Gtk::AccelKey("Delete"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_remove_virtual_device));
 
 		// ---
 		action = Gtk::Action::create(APP_ACTION_NAME(action_add_device), Gtk::Stock::OPEN, "_Add Device...",
 				"Manually add device to device list");
-		actiongroup_main->add(action, Gtk::AccelKey("<control>D"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_add_device), action));
+		actiongroup_main->add((action_map[action_add_device] = action), Gtk::AccelKey("<control>D"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_add_device));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_load_virtual), Gtk::Stock::OPEN, "L_oad Smartctl Output as Virtual Device...",
 				"Load smartctl output from a text file as a read-only virtual device");
-		actiongroup_main->add(action, Gtk::AccelKey("<control>O"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_load_virtual), action));
+		actiongroup_main->add((action_map[action_load_virtual] = action), Gtk::AccelKey("<control>O"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_load_virtual));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_rescan_devices), Gtk::Stock::REFRESH, "_Re-scan Device List",
 				"Re-scan device list");
-		actiongroup_main->add(action, Gtk::AccelKey("<control>R"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_rescan_devices), action));
+		actiongroup_main->add((action_map[action_rescan_devices] = action), Gtk::AccelKey("<control>R"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_rescan_devices));
 
 	actiongroup_main->add(Gtk::Action::create("options_menu", "_Options"));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_executor_log), "View Execution Log");
-		actiongroup_main->add(action,
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_executor_log), action));
+		actiongroup_main->add((action_map[action_executor_log] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_executor_log));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_preferences), Gtk::Stock::PREFERENCES);
-		actiongroup_main->add(action, Gtk::AccelKey("<alt>P"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_preferences), action));
+		actiongroup_main->add((action_map[action_preferences] = action), Gtk::AccelKey("<alt>P"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_preferences));
 
 	actiongroup_main->add(Gtk::Action::create("help_menu", "_Help"));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_general_help), Gtk::Stock::HELP, "A_ll Help Topics");
-		actiongroup_main->add(action, Gtk::AccelKey("F1"),
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_general_help), action));
+		actiongroup_main->add((action_map[action_general_help] = action), Gtk::AccelKey("F1"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_general_help));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_permission_problems), "Resolving _Permission Problems");
-		actiongroup_main->add(action,
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_permission_problems), action));
+		actiongroup_main->add((action_map[action_permission_problems] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_permission_problems));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_how_to_enable_smart), "How to _Enable SMART Permanently");
-		actiongroup_main->add(action,
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_how_to_enable_smart), action));
+		actiongroup_main->add((action_map[action_how_to_enable_smart] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_how_to_enable_smart));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_report_bug), "_Reporting Bugs");
-		actiongroup_main->add(action,
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_report_bug), action));
+		actiongroup_main->add((action_map[action_report_bug] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_report_bug));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_about), Gtk::Stock::ABOUT);
-		actiongroup_main->add(action,
-				sigc::bind(sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_about), action));
+		actiongroup_main->add((action_map[action_about] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_about));
 
 
 
@@ -347,7 +369,7 @@ bool GscMainWindow::create_widgets()
 	}
 	catch(Glib::Error& ex)
 	{
-		debug_out_error("app", DBG_FUNC_MSG << "Create failed: " << ex.what() << "\n");
+		debug_out_error("app", DBG_FUNC_MSG << "UI creation failed: " << ex.what() << "\n");
 		return false;
 	}
 
@@ -414,19 +436,61 @@ bool GscMainWindow::create_widgets()
 
 
 
-void GscMainWindow::on_action_activated(Glib::RefPtr<Gtk::Action> action, GscMainWindow::action_t action_type)
+namespace {
+
+	/// Return true if the user agrees to quit
+	inline bool ask_about_quit_on_test(Gtk::Window& parent)
+	{
+		int status = 0;
+		{
+			Gtk::MessageDialog dialog(parent,
+					"\nOne of the drives is performing a test. Do you really want to quit?\n\n"
+					"<small>The test will continue to run in the background, but you won't be"
+					" able to monitor it using GSmartControl.</small>",
+					true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+			status = dialog.run();
+		}
+		return (status == Gtk::RESPONSE_YES);
+	}
+	
+}
+
+
+
+
+// NOTE: Do NOT bind the Glib::RefPtr<Gtk::Action> parameter to this function.
+// Doing so causes valgrind errors on window destroy (and Send Report dialogs on win32).
+// Probably a gtkmm bug. Use action maps or Gtk::Action* instead.
+void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 {
 	if (!this->action_handling_enabled_)  // check if we should do something
 		return;
 
-	debug_out_info("app", DBG_FUNC_MSG << "Action activated: \"" << action->get_name() << "\"\n");
+	if (action_map.find(action_type) == action_map.end()) {
+		debug_out_error("app", DBG_FUNC_MSG << "Invalid action activated: " << static_cast<int>(action_type) << ".\n");
+		return;
+	}
+
+	Glib::RefPtr<Gtk::Action> action = action_map[action_type];
+	if (!action) {
+		debug_out_error("app", DBG_FUNC_MSG << "Action is NULL for action type " << static_cast<int>(action_type) << ".\n");
+		return;
+	}
+
+	std::string action_name = action->get_name();
+
+	// Do NOT output action->get_name() directly, it dies with unhandled conversion error
+	// exception if used with operator <<.
+	debug_out_info("app", DBG_FUNC_MSG << "Action activated: \"" + action_name << "\"\n");
+
 
 	switch (action_type) {
 		case action_quit:
 			// if at least one drive is having a test performed, disallow.
 			if (this->testing_active()) {
-				gui_show_warn_dialog("Please wait until all tests are finished.", this);
-				break;
+				if (!ask_about_quit_on_test(*this)) {
+					break;
+				}
 			}
 			app_quit();  // ends the main loop
 			break;
@@ -549,7 +613,7 @@ void GscMainWindow::on_action_activated(Glib::RefPtr<Gtk::Action> action, GscMai
 		}
 
 		default:
-			debug_out_error("app", DBG_FUNC_MSG << "Unknown action: \"" << action->get_name() << "\"\n");
+			debug_out_error("app", DBG_FUNC_MSG << "Unknown action: \"" << action_name << "\"\n");
 			break;
 	}
 }
@@ -855,7 +919,7 @@ void GscMainWindow::update_status_widgets()
 		return;
 	}
 
-	std::string device = (drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device());
+	std::string device = (drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device_with_type());
 	std::string size = drive->get_device_size_str();
 	std::string model = (drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
 	std::string family = (drive->get_family_name().empty() ? "Unknown" : drive->get_family_name());
@@ -874,7 +938,7 @@ void GscMainWindow::update_status_widgets()
 		if (health_prop.generic_name == "overall_health") {
 			health_label->set_text(health_prop.format_value());
 			std::string fg;
-			if (app_property_get_label_hilight_color(health_prop.warning, fg)) {
+			if (app_property_get_label_highlight_color(health_prop.warning, fg)) {
 				health_label->set_markup("<span color=\"" + fg + "\">"+ health_label->get_text() + "</span>");
 			}
 			// don't set description tooltip - we already have the basic one.
@@ -908,15 +972,29 @@ void GscMainWindow::update_status_widgets()
 
 void GscMainWindow::rescan_devices()
 {
+	// ignore double-scan (may happen because we use gtk loop iterations here).
+	if (this->scanning_)
+		return;
+
 	// don't manipulate window sensitiveness here - it breaks things
 	// (cursors, gtk errors pop out, etc...)
 
 	// if at least one drive is having a test performed, disallow.
 	if (this->testing_active()) {
-		gui_show_warn_dialog("Please wait until all tests are finished.", this);
-		return;
+		int status = 0;
+		{
+			Gtk::MessageDialog dialog(*this,
+					"\nThis operation may abort any running tests. Do you wish to continue?",
+					true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+			status = dialog.run();
+		}
+		if (status != Gtk::RESPONSE_YES) {
+			return;
+		}
 	}
 
+	this->scanning_ = true;
+
 // 	std::string match_str = rconfig::get_data<std::string>("system/device_match_patterns");
 	std::string blacklist_str = rconfig::get_data<std::string>("system/device_blacklist_patterns");
 
@@ -939,10 +1017,9 @@ void GscMainWindow::rescan_devices()
 	sd.add_blacklist_patterns(blacklist_patterns);
 
 
-	SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-	ex->create_running_dialog(this);  // pass this as parent
+	ExecutorFactoryRefPtr ex_factory(new ExecutorFactory(true, this));  // run it with GUI support
 
-	std::string error_msg = sd.detect_and_fetch_basic_data(drives, ex);  // run it with GUI support
+	std::string error_msg = sd.detect_and_fetch_basic_data(drives, ex_factory);
 
 	bool error = false;
 
@@ -963,7 +1040,7 @@ void GscMainWindow::rescan_devices()
 	if (!error && !error_msg.empty()) {  // generic scan error. smartctl errors are not reported during scan at all.
 		// we don't show output button here
 		gsc_executor_error_dialog_show("An error occurred while scanning the system",
-				error_msg, this, false);
+				error_msg, this, false, false);
 		error = true;
 
 	// add them anyway, in case the error was only on one drive.
@@ -984,11 +1061,13 @@ void GscMainWindow::rescan_devices()
 	// in case there are no drives in the system.
 	if (iconview->num_icons == 0)
 		iconview->empty_view_message = GscMainWindowIconView::message_no_drives_found;
+
+	this->scanning_ = false;
 }
 
 
 
-bool GscMainWindow::add_device(const std::string& file)
+bool GscMainWindow::add_device(const std::string& file, const std::string& type_arg, const std::string& extra_args)
 {
 #ifndef _WIN32  	// win32 doesn't have device files, so skip the check
 	hz::File f(file);
@@ -1001,16 +1080,17 @@ bool GscMainWindow::add_device(const std::string& file)
 #endif
 
 	StorageDeviceRefPtr d(new StorageDevice(file));
+	d->set_type_argument(type_arg);
+	d->set_extra_arguments(extra_args);
 	d->set_is_manually_added(true);
 
-	SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-	ex->create_running_dialog(this);  // pass this as parent
+	ExecutorFactoryRefPtr ex_factory(new ExecutorFactory(true, this));  // pass this as dialog parent
 
 	std::vector<StorageDeviceRefPtr> tmp_drives;
 	tmp_drives.push_back(d);
 
 	StorageDetector sd;
-	std::string error_msg = sd.fetch_basic_data(tmp_drives, ex, true);  // return its first error
+	std::string error_msg = sd.fetch_basic_data(tmp_drives, ex_factory, true);  // return its first error
 	if (!error_msg.empty()) {
 		gsc_executor_error_dialog_show("An error occurred while adding the device", error_msg, this);
 
@@ -1096,7 +1176,7 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 		if (status == Gtk::RESPONSE_YES) {
 			SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-			ex->create_running_dialog(this, "Running %s on " + drive->get_device() + "...");
+			ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 			std::string error_msg = drive->set_smart_enabled(true, ex);  // run it with GUI support
 
 			if (!error_msg.empty()) {
@@ -1110,7 +1190,7 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 	// Parse non-virtual drives here.
 	if (!drive->get_is_virtual()) {
 		SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-		ex->create_running_dialog(this, "Running %s on " + drive->get_device() + "...");
+		ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 		std::string error_msg = drive->fetch_data_and_parse(ex);  // run it with GUI support
 
 		if (!error_msg.empty()) {
@@ -1124,7 +1204,8 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 	// usb devices), only very basic info is available and there's no point
 	// in showing this window. - for both virtual and non-virtual.
 	if (!drive->get_fully_parsed()) {
-		gui_show_warn_dialog("No additional information is available for this drive.", this);
+		gsc_no_info_dialog_show("No additional information is available for this drive.",
+				"", this, false, drive->get_info_output(), "Smartctl Output", drive->get_save_filename());
 		return 0;
 	}
 
@@ -1146,66 +1227,10 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 void GscMainWindow::show_add_device_chooser()
 {
-#ifdef _WIN32
-
-	// A simple text entry dialog, since there are no device files on win32.
-
-	static std::string last_str;
-	if (last_str.empty())
-		last_str = "pd0";
-
-	std::string dev;
-	if (gui_show_text_entry_dialog("Add Device", "Enter a device name",
-			"For example, pd0 means the first physical drive",
-			dev, last_str, this, false)) {
-		last_str = dev;  // safe for the future
-		this->add_device(dev);
-	}
-
-
-#else  // non-win32:
-
-	// Show a file chooser
-
-	static std::string last_dir;
-	if (last_dir.empty())
-		last_dir = "/dev";
-
-	Gtk::FileChooserDialog dialog(*this, "Choose Device File...",
-			Gtk::FILE_CHOOSER_ACTION_OPEN);
-
-	// Add response buttons the the dialog
-	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
-
-	if (!last_dir.empty())
-		dialog.set_current_folder(last_dir);
-
-	// Show the dialog and wait for a user response
-	int result = dialog.run();  // the main cycle blocks here
-
-	// Handle the response
-	switch (result) {
-		case Gtk::RESPONSE_ACCEPT:
-		{
-			last_dir = dialog.get_current_folder();  // safe for the future
-
-			std::string file = dialog.get_filename();
-			this->add_device(file);
-			break;
-		}
-
-		case Gtk::RESPONSE_CANCEL: case Gtk::RESPONSE_DELETE_EVENT:
-			// nothing, the dialog is closed already
-			break;
-
-		default:
-			debug_out_error("app", DBG_FUNC_MSG << "Unknown dialog response code: " << result << ".\n");
-			break;
-	}
-
-#endif  // non-win32
-
+	GscAddDeviceWindow* window = GscAddDeviceWindow::create();
+	window->set_main_window(this);
+	window->set_transient_for(*this);
+	window->show();
 }
 
 
@@ -1233,7 +1258,7 @@ void GscMainWindow::show_load_virtual_file_chooser()
 		{
 			last_dir = dialog.get_current_folder();  // safe for the future
 
-			std::string file = dialog.get_filename();
+			std::string file = dialog.get_filename();  // in fs encoding
 			this->add_virtual_drive(file);
 			break;
 		}
@@ -1256,8 +1281,9 @@ bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
 {
 	// if at least one drive is having a test performed, disallow.
 	if (this->testing_active()) {
-		gui_show_warn_dialog("Please wait until all tests are finished.", this);
-		return true;  // handled
+		if (!ask_about_quit_on_test(*this)) {
+			return true;  // handled
+		}
 	}
 	app_quit();  // ends the main loop
 	return true;  // event handled, don't call default virtual handler
