@@ -1,6 +1,6 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
@@ -17,8 +17,10 @@
 #include <gdk/gdkkeysyms.h>  // GDK_Escape
 
 #include "hz/fs_path.h"
+#include "hz/string_sprintf.h"
 #include "rconfig/rconfig_mini.h"
 #include "applib/storage_settings.h"
+#include "applib/app_gtkmm_utils.h"
 
 #include "gsc_preferences_window.h"
 
@@ -36,16 +38,20 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		{
 			Gtk::TreeModelColumnRecord model_columns;
 
-			// Device, [Parameters], [Device Real].
+			// Device, Type, [Parameters], [Device Real], [Type Real].
 			// Device may hold "<empty>", while Device Real is "".
+			// Type may hold "<all>", while Type Real is "".
 
 			model_columns.add(col_device);
 			this->append_column("Device", col_device);
 			this->set_search_column(col_device.index());
 
-			model_columns.add(col_parameters);
+			model_columns.add(col_type);
+			this->append_column("Type", col_type);
 
+			model_columns.add(col_parameters);
 			model_columns.add(col_device_real);
+			model_columns.add(col_type_real);
 
 
 			// create a TreeModel (ListStore)
@@ -76,12 +82,14 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		}
 
 
-		void add_new_row(const std::string& device, const std::string& params, bool select = true)
+		void add_new_row(const std::string& device, const std::string& type, const std::string& params, bool select = true)
 		{
 			Gtk::TreeRow row = *(model->append());
 			row[col_device] = (device.empty() ? "<empty>" : device);
+			row[col_type] = (type.empty() ? "<all>" : type);
 			row[col_parameters] = params;
 			row[col_device_real] = device;
+			row[col_type_real] = type;
 
 			if (select)
 				this->get_selection()->select(row);
@@ -98,6 +106,16 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		}
 
 
+		void update_selected_row_type(const std::string& type)
+		{
+			if (this->get_selection()->count_selected_rows()) {
+				Gtk::TreeRow row = *(this->get_selection()->get_selected());
+				row[col_type] = (type.empty() ? "<all>" : type);
+				row[col_type_real] = type;
+			}
+		}
+
+
 		void update_selected_row_params(const std::string& params)
 		{
 			if (this->get_selection()->count_selected_rows()) {
@@ -123,7 +141,12 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		{
 			clear_all();
 			for (device_option_map_t::const_iterator iter = devmap.begin(); iter != devmap.end(); ++iter) {
-				this->add_new_row(iter->first, iter->second, false);
+				std::vector<std::string> parts;
+				hz::string_split(iter->first, "::", parts, 2);
+				std::string dev = (parts.size() > 0 ? parts.at(0) : "");
+				std::string type = (parts.size() > 1 ? parts.at(1) : "");
+				std::string params = iter->second;
+				this->add_new_row(dev, type, params, false);
 			}
 		}
 
@@ -135,8 +158,16 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 			Gtk::TreeNodeChildren children = model->children();
 			for (Gtk::TreeNodeChildren::iterator iter = children.begin(); iter != children.end(); ++iter) {
 				Gtk::TreeModel::Row row = *iter;
-				if (devmap.find(row[col_device_real]) == devmap.end())
-					devmap[row[col_device_real]] = row[col_parameters];
+				std::string dev = row.get_value(col_device_real);
+				if (!dev.empty()) {
+					std::string par = row.get_value(col_type_real);
+					if (!par.empty()) {
+						dev += "::" + par;
+					}
+					if (devmap.find(dev) == devmap.end()) {
+						devmap[dev] = row.get_value(col_parameters);
+					}
+				}
 			}
 
 			return devmap;
@@ -147,10 +178,11 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		// callback
 		void on_selection_changed()
 		{
-			std::string dev, par;
+			std::string dev, type, par;
 			if (this->get_selection()->count_selected_rows()) {
 				Gtk::TreeRow row = *(this->get_selection()->get_selected());
 				dev = row[col_device_real];
+				type = row[col_type_real];
 				par = row[col_parameters];
 				preferences_window->device_widget_set_remove_possible(true);
 
@@ -158,7 +190,7 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 				preferences_window->device_widget_set_remove_possible(false);
 			}
 
-			preferences_window->update_device_widgets(dev, par);
+			preferences_window->update_device_widgets(dev, type, par);
 		}
 
 
@@ -166,8 +198,10 @@ class GscPreferencesDeviceOptionsTreeView : public Gtk::TreeView {
 		Glib::RefPtr<Gtk::ListStore> model;
 
 		Gtk::TreeModelColumn<Glib::ustring> col_device;
+		Gtk::TreeModelColumn<Glib::ustring> col_type;
 		Gtk::TreeModelColumn<std::string> col_parameters;
 		Gtk::TreeModelColumn<std::string> col_device_real;
+		Gtk::TreeModelColumn<std::string> col_type_real;
 
 		GscPreferencesWindow* preferences_window;
 
@@ -206,9 +240,39 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 	Gtk::Button* device_options_remove_device_button = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_remove_device_button, clicked);
 
+
 	Gtk::Entry* device_options_device_entry = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_device_entry, changed);
 
+	Glib::ustring device_options_tooltip = "Device name";
+#if defined CONFIG_KERNEL_FAMILY_WINDOWS
+	device_options_tooltip = "Device name (for example, use \"pd0\" for the first physical drive)";
+#elif defined CONFIG_KERNEL_LINUX
+	device_options_tooltip = "Device name (for example, /dev/sda or /dev/twa0)";
+#endif
+	if (Gtk::Label* device_options_device_label = lookup_widget<Gtk::Label*>("device_options_device_label")) {
+		app_gtkmm_set_widget_tooltip(*device_options_device_label, device_options_tooltip);
+	}
+	if (device_options_device_entry) {
+		app_gtkmm_set_widget_tooltip(*device_options_device_entry, device_options_tooltip);
+	}
+
+
+	Gtk::Entry* device_options_type_entry = 0;
+	APP_UI_RES_AUTO_CONNECT(device_options_type_entry, changed);
+
+	Glib::ustring device_type_tooltip = "Match only this type of device (as specified to -d smartctl parameter)";
+#if defined CONFIG_KERNEL_LINUX
+	device_type_tooltip = "Match only this type of device (as specified to -d smartctl parameter). Leave empty for all types. This can be used to specify a drive behind a RAID device, e.g. \"3ware,2\".";
+#endif
+	if (Gtk::Label* device_options_type_label = lookup_widget<Gtk::Label*>("device_options_type_label")) {
+		app_gtkmm_set_widget_tooltip(*device_options_type_label, device_type_tooltip);
+	}
+	if (device_options_type_entry) {
+		app_gtkmm_set_widget_tooltip(*device_options_type_entry, device_type_tooltip);
+	}
+
+
 	Gtk::Entry* device_options_parameter_entry = 0;
 	APP_UI_RES_AUTO_CONNECT(device_options_parameter_entry, changed);
 
@@ -229,6 +293,13 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 	// we can't do this in treeview's constructor, it doesn't know about this window yet.
 	this->device_widget_set_remove_possible(false);  // initial state
 
+	// hide win32-only options for non-win32.
+#ifndef _WIN32
+	Gtk::CheckButton* smartctl_search_check = this->lookup_widget<Gtk::CheckButton*>("search_in_smartmontools_first_check");
+	if (smartctl_search_check)
+		smartctl_search_check->hide();
+#endif
+
 
 	// ---------------
 
@@ -240,13 +311,16 @@ GscPreferencesWindow::GscPreferencesWindow(BaseObjectType* gtkcobj, const app_ui
 
 
 
-void GscPreferencesWindow::update_device_widgets(const std::string& device, const std::string& params)
+void GscPreferencesWindow::update_device_widgets(const std::string& device, const std::string& type, const std::string& params)
 {
 	Gtk::Entry* entry = 0;
 
 	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_device_entry")))
 		entry->set_text(device);
 
+	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_type_entry")))
+		entry->set_text(type);
+
 	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_parameter_entry")))
 		entry->set_text(params);
 }
@@ -308,16 +382,21 @@ void GscPreferencesWindow::import_config()
 
 	// ------- General tab
 
-	bool scan_on_startup;
+	bool scan_on_startup = 0;
 	if ( prefs_config_get("gui/scan_on_startup", scan_on_startup)
 			&& (check = this->lookup_widget<Gtk::CheckButton*>("scan_on_startup_check")) )
 		check->set_active(scan_on_startup);
 
-	bool show_smart_capable_only;
+	bool show_smart_capable_only = 0;
 	if ( prefs_config_get("gui/show_smart_capable_only", show_smart_capable_only)
 			&& (check = this->lookup_widget<Gtk::CheckButton*>("show_smart_capable_only_check")) )
 		check->set_active(show_smart_capable_only);
 
+	bool win32_search_smartctl_in_smartmontools = 0;
+	if ( prefs_config_get("system/win32_search_smartctl_in_smartmontools", win32_search_smartctl_in_smartmontools)
+			&& (check = this->lookup_widget<Gtk::CheckButton*>("search_in_smartmontools_first_check")) )
+		check->set_active(win32_search_smartctl_in_smartmontools);
+
 	std::string smartctl_binary;
 	if ( prefs_config_get("system/smartctl_binary", smartctl_binary)
 			&& (entry = this->lookup_widget<Gtk::Entry*>("smartctl_binary_entry")) )
@@ -363,6 +442,9 @@ void GscPreferencesWindow::export_config()
 	if ((check = this->lookup_widget<Gtk::CheckButton*>("show_smart_capable_only_check")))
 		prefs_config_set("gui/show_smart_capable_only", bool(check->get_active()));
 
+	if ((check = this->lookup_widget<Gtk::CheckButton*>("search_in_smartmontools_first_check")))
+		prefs_config_set("system/win32_search_smartctl_in_smartmontools", bool(check->get_active()));
+
 	if ((entry = this->lookup_widget<Gtk::Entry*>("smartctl_binary_entry")))
 		prefs_config_set("system/smartctl_binary", std::string(entry->get_text()));
 
@@ -454,18 +536,21 @@ void GscPreferencesWindow::on_device_options_remove_device_button_clicked()
 void GscPreferencesWindow::on_device_options_add_device_button_clicked()
 {
 	Gtk::Entry* entry = 0;
-	std::string dev, par;
+	std::string dev, type, par;
 
 	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_device_entry")))
 		dev = entry->get_text();
 
+	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_type_entry")))
+		type = entry->get_text();
+
 	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_parameter_entry")))
 		par = entry->get_text();
 
 	if (device_options_treeview->has_selected_row()) {
-		device_options_treeview->add_new_row("", "");  // without this it would clone the existing.
+		device_options_treeview->add_new_row("", "", "");  // without this it would clone the existing.
 	} else {
-		device_options_treeview->add_new_row(dev, par);
+		device_options_treeview->add_new_row(dev, type, par);
 	}
 }
 
@@ -480,6 +565,15 @@ void GscPreferencesWindow::on_device_options_device_entry_changed()
 
 
 
+void GscPreferencesWindow::on_device_options_type_entry_changed()
+{
+	Gtk::Entry* entry = 0;
+	if ((entry = this->lookup_widget<Gtk::Entry*>("device_options_type_entry")))
+		device_options_treeview->update_selected_row_type(entry->get_text());
+}
+
+
+
 void GscPreferencesWindow::on_device_options_parameter_entry_changed()
 {
 	Gtk::Entry* entry = 0;
