@@ -1,6 +1,6 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
 
@@ -35,7 +35,7 @@ class GscPreferencesWindow : public AppUIResWidget<GscPreferencesWindow, true> {
 		{ }
 
 
-		void update_device_widgets(const std::string& device, const std::string& params);
+		void update_device_widgets(const std::string& device, const std::string& type, const std::string& params);
 
 		void device_widget_set_remove_possible(bool b);
 
@@ -90,6 +90,8 @@ class GscPreferencesWindow : public AppUIResWidget<GscPreferencesWindow, true> {
 
 		void on_device_options_device_entry_changed();
 
+		void on_device_options_type_entry_changed();
+
 		void on_device_options_parameter_entry_changed();
 
 
