@@ -1,7 +1,7 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_whatever.txt
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
+ License: See LICENSE_unlicense.txt
 ***************************************************************************/
 
 // disable libdebug, we don't link to it
@@ -33,7 +33,7 @@ int main()
 
 	{
 		std::string s;
-		hz::string_format(s, "abc %s efg %d ijk %Lf lmn %s\n")("hello")(5)(8.99)(std::string("werld"));
+		hz::string_format(s, "abc %s efg %d ijk %f lmn %s\n")("hello")(5)(8.99)(std::string("werld"));
 		std::cerr << s;
 	}
 
