@@ -1,13 +1,14 @@
 /**************************************************************************
  Copyright:
-      (C) 2008  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2011  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
 
 #ifndef LIBDEBUG_DEXCEPT_H
 #define LIBDEBUG_DEXCEPT_H
 
-#include <cstring>  // strncpy / strlen
+#include <cstddef>  // std::size_t
+#include <cstring>  // std::strncpy / std::strlen
 #include <exception>  // std::exception
 
 
